﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once

namespace s3d
{
	//----------------
	//	SivOptional.hpp
	
	template <typename Type> class Optional;

	//----------------
	//	SivChar.hpp

	struct Char;

	//----------------
	//	SivString.hpp

	class String;
	using FilePath = String;

	//----------------
	//	SivStringView.hpp

	class StringView;

	//----------------
	//	SivInteger.hpp

	template <typename Type> struct IntegerBase;
	using Integer8		= IntegerBase<char>;
	using IntegerU8		= IntegerBase<unsigned __int8>;
	using Integer16		= IntegerBase<__int16>;
	using IntegerU16	= IntegerBase<unsigned __int16>;
	using Integer32		= IntegerBase<__int32>;
	using Integer		= Integer32;
	using IntegerU32	= IntegerBase<unsigned __int32>;
	using Integer64		= IntegerBase<__int64>;
	using IntegerU64	= IntegerBase<unsigned __int64>;

	//----------------
	//	SivFloat.hpp

	template <typename Type, typename BitsType> struct FloatBase;
	using Float		= FloatBase<float, unsigned __int32>;
	using Double	= FloatBase<double, unsigned __int64>;

	//----------------
	//	SivGrid.hpp

	template<typename Type> class Grid;

	//----------------
	//	SivDate.hpp

	struct Date;

	//----------------
	//	SivDateTime.hpp

	struct DateTime;

	//----------------
	//	SivTimer.hpp

	template <int resolutionMilliseconds> class TimerBase;
	using TimerMillisec	= TimerBase<1>;
	using TimerSec		= TimerBase<1000>;
	using TimerMinute	= TimerBase<1000 * 60>;
	class TimerMicrosec;

	//----------------
	//	SivEventTimer.hpp

	template<typename Timer> class EventTimer;
	using EventTimerMillisec	= EventTimer<TimerMillisec>;
	using EventTimerSec			= EventTimer<TimerSec>;
	using EventTimerMinute		= EventTimer<TimerMinute>;

	//----------------
	//	SivProfilerUtility.hpp

	class MillisecClock;
	class MicrosecClock;
	class RDTSCClock;

	//----------------
	//	SivFileSystem.hpp

	enum class OpenMode;
	enum class TextEncoding;
	enum class CopyOption;

	//----------------
	//	SivIReader.hpp

	class IReader;

	//----------------
	//	SivMemoryReader.hpp

	class MemoryReader;

	//----------------
	//	SivBinaryReader.hpp

	class BinaryReader;

	//----------------
	//	SivIWriter.hpp

	class IWriter;

	//----------------
	//	SivMemoryWriter.hpp

	class MemoryWriter;

	//----------------
	//	SivBinaryWriter.hpp

	class BinaryWriter;

	//----------------
	//	SivTextReader.hpp

	class TextReader;

	//----------------
	//	SivTextWriter.hpp

	class TextWriter;

	//----------------
	//	SivLogger.hpp

	enum class OutputLevel;
	enum class LogDescription;

	//----------------
	//	SivCSVReader.hpp

	class CSVReader;

	//----------------
	//	SivCSVWriter.hpp

	class CSVWriter;

	//----------------
	//	SivINIReader.hpp

	class INIReader;

	//----------------
	//	SivINIWriter.hpp

	class INIWriter;

	//----------------
	//	SivXMLReader.hpp

	class XMLElement;
	class XMLReader;

	//----------------
	//	SivColor.hpp

	union Color;
	struct ColorF;

	//----------------
	//	SivHSV..hpp

	struct HSV;

	//----------------
	//	SivPointVector.hpp

	struct  Point;
	template<typename Type> struct Vector2D;
	template<typename Type> struct Vector3D;
	template<typename Type> struct Vector4D;
	using Float2	= Vector2D<float>;
	using Vec2		= Vector2D<double>;
	using Float3	= Vector3D<float>;
	using Vec3		= Vector3D<double>;
	using Float4	= Vector4D<float>;
	using Vec4		= Vector4D<double>;

	//----------------
	//	SivCircular.hpp

	template<int Oclock> struct  CircularBase;
	using Circular	= CircularBase<0>;
	using Circular0	= CircularBase<0>;
	using Circular3 = CircularBase<3>;
	using Circular6 = CircularBase<6>;
	using Circular9 = CircularBase<9>;

	//----------------
	//	SivLine.hpp

	struct  LineInt;
	struct  Line;

	//----------------
	//	SivRectangle.hpp

	template<typename SizeType, typename ElementType> struct Rectangle;
	using Rect	= Rectangle<Point, int>;
	using RectF	= Rectangle<Vec2, double>;

	//----------------
	//	SivCircle.hpp

	struct Circle;

	//----------------
	//	SivEllipse.hpp

	struct Ellipse;

	//----------------
	//	SivTriangle.hpp

	struct Triangle;

	//----------------
	//	SivQuad.hpp

	struct Quad;

	//----------------
	//	SivRoundRect.hpp

	struct RoundRect;

	//----------------
	//	SivLineString.hpp

	class LineString;

	//----------------
	//	SivPolygon.hpp

	class PolygonHoles;
	class Polygon;
	class PolygonTriangles;

	//----------------
	//	SivMultiPolygon.hpp

	class MultiPolygon;

	//----------------
	//	SivRandom.hpp

	class RNG;

	//----------------
	//	SivNoise.hpp

	class Noise;

	//----------------
	//	SivSIMD.hpp

	enum class SSE;

	//----------------
	//	SivImage.hpp

	enum class ImageFormat;
	enum class DDSFormat;
	enum class WebPMethod;
	enum class Interpolation;
	enum class ImageAddressMode;
	enum class AdaptiveMethod;
	enum class Interpolation;
	enum class FloodFillConnectivity;
	enum class CascadeType;
	class Image;
	class ImageRegion;

	//----------------
	//	SivTextureFormat.hpp

	enum class TextureFormat;

	//----------------
	//	SivCustomColor.hpp

	struct R32F;
	union RGBA16F;
	union RGBA32F;

	//----------------
	//	SivCustomImage.hpp

	template<typename Type> class CustomImage;
	using ImageR32F		= CustomImage<R32F>;
	using ImageRGBA16F	= CustomImage<RGBA16F>;
	using ImageRGBA32F	= CustomImage<RGBA32F>;

	//----------------
	//	SivWaveSample.hpp

	union WaveSample;

	//----------------
	//	SivWave.hpp

	enum class AudioFormat;
	struct AudioProperty;
	enum class MP3Quality;
	enum class AACQuality;
	class Wave;

	//----------------
	//	SivFFT.hpp

	enum class FFTSampleLength;
	class FFTResult;

	//----------------
	//	SivVideoWriter.hpp

	enum class VideoCompression;
	class VideoWriter;

	//----------------
	//	SivVideoReader.hpp

	class VideoReader;

	//----------------
	//	SivSerial.hpp

	class Serial;
	struct SerialDeveiceInfo;

	//----------------
	//	SivInternet.hpp

	enum class DownloadStatus;
	struct FileDownloadState;

	//----------------
	//	SivTwitter.hpp

	namespace experimental
	{
		class TwitterClient;
	}

	//----------------
	//	SivWindow.hpp

	enum class WindowStyle;
	struct WindowState;

	//----------------
	//	SivMessageBox.hpp

	enum class MessageBoxStyle;
	enum class MessageBoxCommand;

	//----------------
	//	SivSystem.hpp

	struct WindowEvent;

	//----------------
	//	SivClipboard.hpp

	enum class ClipboardItemType;

	//----------------
	//	SivDialog.hpp

	enum class ExtensionFilter;
	struct ExtensionFilterPair;

	//----------------
	//	SivProfiler.hpp

	struct GraphicsProfile;

	//----------------
	//	SivRecorder.hpp

	namespace RecorderFormat
	{
		enum Type;
	}

	//----------------
	//	SivKey.hpp

	class Key;
	class PlusCombinedKeys;
	class OrCombinedKeys;

	//----------------
	//	SivTouch.hpp

	struct Touch;

	//----------------
	//	SivGamepad.hpp

	struct GamepadInfo;
	enum class DeadZoneType;
	struct DeadZone;
	class GamepadButton;
	class Gamepad;

	//----------------
	//	SivXInput.hpp

	class XInput;

	//----------------
	//	SivKinectV1.hpp

	enum class TrackingState;
	struct KinectV1Joint;
	struct KinectV1Body;

	//----------------
	//	SivKinectV2.hpp

	enum class HandState;
	struct KinectV2Joint;
	struct KinectV2Body;
	struct KinectV2AudioBeam;

	//----------------
	//	SivSound.hpp

	struct SoundLoop;
	class Sound;

	//----------------
	//	SivBlendState.hpp

	enum class Blend : unsigned char;
	enum class BlendOp : unsigned char;
	struct BlendState;

	//----------------
	//	SivRasterizerState.hpp

	enum class FillMode : unsigned char;
	enum class CullMode : unsigned char;
	struct RasterizerState;

	//----------------
	//	SivDepthStencilState.hpp

	enum class DepthFunc : unsigned short;
	struct DepthState;
	enum class StencilOp : unsigned char;
	enum class StencilFunc : unsigned char;
	struct StencilState;
	struct DepthStencilState;

	//----------------
	//	SivSamplerState.hpp

	enum class TextureFilter : unsigned char;
	enum class TextureAddressMode : unsigned char;
	struct SamplerState;

	//----------------
	//	SivRenderState.hpp

	struct RenderState;

	//----------------
	//	SivVertexShader.hpp

	class VertexShader;

	//----------------
	//	SivPixelShader.hpp

	class PixelShader;

	//----------------
	//	SivConstantBuffer.hpp

	enum class ShaderStage;
	class ConstantBufferBase;
	template<typename Type> class ConstantBuffer;

	//----------------
	//	SivInputLayout.hpp

	enum class InputClassification;
	struct InputElementDesc;
	class InputLayout;

	//----------------
	//	SivTexture.hpp

	enum class TextureDesc;
	class Texture;

	//----------------
	//	SivTextureRegion.hpp

	struct TextureRegion;

	//----------------
	//	SivTexturedQuad.hpp

	struct TexturedQuad;

	//----------------
	//	SivDynamicTexture.hpp

	class DynamicTexture;

	//----------------
	//	SivSprite.hpp

	struct SpriteVertex;
	class Sprite;

	//----------------
	//	SivKineticTypography.hpp

	struct KineticTypography;

	//----------------
	//	SivFont.hpp

	enum class FontStyle;
	struct FontProperty;
	class Font;
	class DrawableString;
	struct TextStyle;

	//----------------
	//	SivWidgetStyle.hpp

	struct Padding;
	struct Margin;
	enum class Align;
	enum class Repeat;
	struct Background;
	struct WidgetStyle;
	struct WidgetState;

	//----------------
	//	SivWidget.hpp

	class IWidget;

	//----------------
	//	SivGUIText.hpp

	class GUIText;
	class GUITextWrapper;

	//----------------
	//	SivGUINewLine.hpp

	class GUINewLine;
	class GUINewLineWrapper;

	//----------------
	//	SivGUITexture.hpp

	class GUITexture;
	class GUITextureWrapper;

	//----------------
	//	SivGUIHorizontalLine.hpp

	class GUIHorizontalLine;
	class GUIHorizontalLineWrapper;

	//----------------
	//	SivGUIButton.hpp

	class GUIButton;
	class GUIButtonWrapper;

	//----------------
	//	SivGUIToggleSwitch.hpp

	class GUIToggleSwitch;
	class GUIToggleSwitchWrapper;

	//----------------
	//	SivGUISlider.hpp

	class GUISlider;
	class GUISliderWrapper;

	//----------------
	//	SivCheckBox.hpp
	
	class GUICheckBox;
	class GUICheckBoxWrapper;

	//----------------
	//	SivGUIRadioButton.hpp

	class GUIRadioButton;
	class GUIRadioButtonWrapper;

	//----------------
	//	SivGUITextField.hpp

	class GUITextField;
	class GUITextFieldWrapper;

	//----------------
	//	SivGUITextArea.hpp

	class GUITextArea;
	class GUITextAreaWrapper;

	//----------------
	//	SivGUI.hpp

	struct GUIStyle;
	class GUI;

	//----------------
	//	SivCursor.hpp

	enum class CursorType;
	struct CursorStyle;

	//----------------
	//	SivWebcam.hpp

	struct WebcamInfo;

	//----------------
	//	SivAR.hpp

	struct ARMarker;

	//----------------
	//	SivQuaternion.hpp

	struct Quaternion;

	//----------------
	//	SivMatrix.hpp

	struct Mat4x4;

	//----------------
	//	SivRay.hpp

	struct Ray;

	//----------------
	//	SivTriangle3D.hpp

	struct Triangle3D;

	//----------------
	//	SivMeshData.hpp

	struct MeshVertex;
	enum class PlaneFace;
	struct MeshData;

	//----------------
	//	SivMesh.hpp

	class Mesh;

	//----------------
	//	SivDynamicMesh.hpp

	class DynamicMesh;

	//----------------
	//	SivTransformedMesh.hpp

	struct TransformedMesh;

	//----------------
	//	SivParticle.hpp

	struct Particle;

	//----------------
	//	SivPlane.hpp

	struct Plane;

	//----------------
	//	SivSphere.hpp

	struct Sphere;

	//----------------
	//	SivBox.hpp

	struct Box;

	//----------------
	//	SivDisc.hpp

	struct Disc;

	//----------------
	//	SivCylinder.hpp

	struct Cylinder;

	//----------------
	//	SivCone.hpp

	struct Cone;

	//----------------
	//	SivModel.hpp

	struct Material;
	struct ModelNode;
	class Model;

	//----------------
	//	SivCamera.hpp

	struct Camera;

	//----------------
	//	SivFog.hpp

	struct Fog;

	//----------------
	//	SivLight.hpp

	struct Light;

	//----------------
	//	SivGraphics3D.hpp

	enum class Renderer;
	enum class AntiAliasing;

	//----------------
	//	SivAsset.hpp

	using AssetName = String;
	using AssetTag = String;
	enum AssetFlag;
	class IAsset;

	//----------------
	//	SivSoundAsset.hpp

	struct SoundAssetData;
	class SoundAsset;

	//----------------
	//	SivTextureAsset.hpp

	struct TextureAssetData;
	class TextureAsset;

	//----------------
	//	SivFontAsset.hpp

	struct FontAssetData;
	class FontAsset;

	//----------------
	//	SivGUIAsset.hpp

	struct GUIAssetData;
	class GUIAsset;

	//----------------


}
