﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "SivFormat.hpp"

namespace s3d
{
	/// <summary>
	/// 画面にテキストを表示します。
	/// </summary>
	/// <param name="text">
	/// テキスト
	/// </param>
	/// <returns>
	/// なし
	/// </returns>
	void Print(const String& text);

	/// <summary>
	/// 画面にテキストを表示し、改行します。
	/// </summary>
	/// <param name="text">
	/// テキスト
	/// </param>
	/// <returns>
	/// なし
	/// </returns>
	void Println(const String& text);

	/// <summary>
	/// 画面にデータを表示します。
	/// </summary>
	/// <param name="args">
	/// テキスト
	/// </param>
	/// <returns>
	/// なし
	/// </returns>
	template <typename ... Args> inline void Print(const Args& ... args)
	{
		Print(Format(args...));
	}

	/// <summary>
	/// 画面にデータを表示し、改行します。
	/// </summary>
	/// <param name="args">
	/// テキスト
	/// </param>
	/// <returns>
	/// なし
	/// </returns>
	template <typename ... Args> inline void Println(const Args& ... args)
	{
		Println(Format(args...));
	}

	/// <summary>
	/// 画面に表示したデータを消去します。
	/// </summary>
	/// <returns>
	/// なし
	/// </returns>
	void ClearPrint();

	/// <summary>
	/// 何らかのキーが入力されるまで待機します。
	/// </summary>
	/// <remarks>
	/// 待機の間、GUI は更新されません。
	/// </remarks>
	void WaitKey();
}
