﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "Siv3DFwd.hpp"
# include "SivString.hpp"
# include "SivArray.hpp"

namespace s3d
{
	/// <summary>
	/// アセット名
	/// </summary>
	using AssetName = String;

	/// <summary>
	/// アセットタグ名
	/// </summary>
	using AssetTag = String;

	/// <summary>
	/// アセット作成フラグ
	/// </summary>
	enum AssetFlag
	{
		/// <summary>
		/// デフォルト
		/// </summary>
		Default = 0x0,

		/// <summary>
		/// 登録時にプリロード
		/// </summary>
		LoadImmediately = 0x1,

		/// <summary>
		/// 元のファイルが更新された時に、アセットの内容を更新しない
		/// </summary>
		Immutable = 0x2,
	};

	/// <summary>
	/// アセットインタフェースクラス
	/// </summary>
	class IAsset
	{
	public:

		/// <summary>
		/// デストラクタ
		/// </summary>
		virtual ~IAsset(){}

		/// <summary>
		/// アセットのプリロード時に呼ばれる関数
		/// </summary>
		/// <returns>
		/// プリロードに成功した場合 true, それ以外の場合は false
		/// </returns>
		virtual bool preload() = 0;

		/// <summary>
		/// アセットの更新時に呼ばれる関数
		/// </summary>
		/// <returns>
		/// 更新に成功した場合 true, それ以外の場合は false
		/// </returns>
		virtual bool update() = 0;

		/// <summary>
		/// アセットのリリース時に呼ばれる関数
		/// </summary>
		/// <returns>
		/// リリースに成功した場合 true, それ以外の場合は false
		/// </returns>
		virtual bool release() = 0;

		/// <summary>
		/// アセットのタグを取得します。
		/// </summary>
		/// <returns>
		/// アセットのタグ一覧
		/// </returns>
		virtual const Array<AssetTag>& getTags() const = 0;
	};
}
