﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "SivPropertyMacro.hpp"

namespace s3d
{
	/// <summary>
	/// 文字
	/// </summary>
	struct Char
	{
		/// <summary>
		/// 文字
		/// </summary>
		unsigned ch;

		/// <summary>
		/// コンストラクタ
		/// </summary>
		Char() {}

		/// <summary>
		/// コンストラクタ
		/// </summary>
		/// <param name="_ch">
		/// コピーする文字
		/// </param>
		Char(unsigned _ch)
			: ch(_ch) {}

		/// <summary>
		/// 文字を wchar_t 型に変換します。
		/// </summary>
		/// <returns>
		/// wchar_t 型の文字
		/// </returns>
		operator wchar_t() const;

		/// <summary>
		/// 10 進数の数字であるかを示します。
		/// </summary>
		Property_Get(bool, isDigit) const;

		/// <summary>
		/// アルファベットの小文字であるかを示します。
		/// </summary>
		Property_Get(bool, isLower) const;

		/// <summary>
		/// アルファベットの大文字であるかを示します。
		/// </summary>
		Property_Get(bool, isUpper) const;

		/// <summary>
		/// アルファベットであるかを示します。
		/// </summary>
		Property_Get(bool, isAlpha) const;

		/// <summary>
		/// アルファベットもしくは数字であるかを示します。
		/// </summary>
		Property_Get(bool, isAlnum) const;

		/// <summary>
		/// 16 進数の数字であるかを示します。
		/// </summary>
		Property_Get(bool, isXdigit) const;

		/// <summary>
		/// 制御文字であるかを示します。
		/// </summary>
		Property_Get(bool, isControl) const;

		/// <summary>
		/// 空白文字であるかを示します。
		/// </summary>
		/// <remarks>
		/// L' ', L'\t', L'　' が該当します。
		/// </remarks>
		Property_Get(bool, isBlank) const;

		/// <summary>
		/// 空白類文字であるかを示します。
		/// </summary>
		/// <remarks>
		/// L' ', L'\t', L'\n', L'\v', L'\f', L'\r', L'　' が該当します。
		/// </remarks>
		Property_Get(bool, isSpace) const;

		/// <summary>
		/// 印字可能文字であるかを示します。
		/// </summary>
		Property_Get(bool, isPrint) const;
	};
}

# include "SivChar.inl"
