﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <iostream>
# include "SivPropertyMacro.hpp"
# include "SivPointVector.hpp"
# include "SivMathConstants.hpp"

namespace s3d
{
	/// <summary>
	/// 極座標
	/// </summary>
	template<int Oclock>
	struct CircularBase
	{
	private:

		double offset() const
		{
			return Oclock * (Math::TwoPi / 12);
		}

	public:

		double r, theta;

		/**
		*
		*/
		CircularBase() {}

		/**
		*
		*	@param _r
		*	@param _theta
		*/
		CircularBase(double _r, double _theta) : r(_r), theta(_theta) {}

		/**
		*
		*	@param pos
		*/
		CircularBase(const Vec2& pos) : r(pos.length()), theta(::atan2(pos.x, -pos.y) - offset()) {}

		/**
		*
		*	@return
		*/
		CircularBase operator + () const
		{
			return *this;
		}

		/**
		*
		*	@return
		*/
		CircularBase operator - () const
		{
			return{ r, theta + Math::Pi };
		}

		/**
		*
		*	@param v
		*	@return
		*/
		Vec2 operator + (const Vec2& v) const
		{
			return operator Vec2() + v;
		}

		/**
		*
		*	@param v
		*	@return
		*/
		Vec2 operator - (const Vec2& v) const
		{
			return operator Vec2() - v;
		}

		/**
		*
		*	@return
		*/
		operator Vec2() const
		{
			return{ ::sin(theta + offset())*r, -::cos(theta + offset())*r };
		}
	};

	/**
	*
	*	@param os
	*	@param c
	*	@return
	*/
	template<typename CharType, int Oclock>
	inline
	std::basic_ostream<CharType>&
		operator << (
		std::basic_ostream<CharType>& os,
		const CircularBase<Oclock>& c
		)
	{
		return os << CharType('(') << c.r << CharType(',') << c.theta << CharType(')');
	}

	/**
	*
	*	@param is
	*	@param c
	*	@return
	*/
	template<typename CharType, int Oclock>
	inline
	std::basic_istream<CharType>&
		operator >> (
		std::basic_istream<CharType>& is,
		CircularBase<Oclock>& c
		)
	{
		CharType unused;
		return is >> unused >> c.r >> unused >> c.theta >> unused;
	}

	/**
	*
	*	@param pos
	*	@param c
	*	@return
	*/
	template<int Oclock>
	inline Vec2 operator + (const Vec2& pos, const CircularBase<Oclock>& c)
	{
		return c + pos;
	}

	/**
	*
	*	@param pos
	*	@param c
	*	@return
	*/
	template<int Oclock>
	inline Vec2 operator - (const Vec2& pos, const CircularBase<Oclock>& c)
	{
		return pos + (-c);
	}

	using Circular	= CircularBase<0>;
	using Circular0	= CircularBase<0>;
	using Circular3 = CircularBase<3>;
	using Circular6 = CircularBase<6>;
	using Circular9 = CircularBase<9>;
}
