﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <iostream>
# include "Siv3DFwd.hpp"
# include "SivPropertyMacro.hpp"
# include "SivStringView.hpp"

namespace s3d
{
	/// <summary>
	/// カラーパレット
	/// </summary>
	struct Palette
	{
		const static Color Black;			// (0,0,0)
		const static Color Dimgray;			// (105,105,105)
		const static Color Gray;			// (128,128,128)
		const static Color Darkgray;		// (169,169,169)
		const static Color Silver;			// (192,192,192)
		const static Color Lightgrey;		// (211,211,211)
		const static Color Gainsboro;		// (220,220,220)
		const static Color Whitesmoke;		// (245,245,245)
		const static Color White;			// (255,255,255)
		const static Color Snow;			// (255,250,250)
		const static Color Ghostwhite;		// (248,248,255)
		const static Color Floralwhite;		// (255,250,240)
		const static Color Linen;			// (250,240,230)
		const static Color Antiquewhite;	// (250,235,215)
		const static Color Papayawhip;		// (255,239,213)
		const static Color Blanchedalmond;	// (255,235,205)
		const static Color Bisque;			// (255,228,196)
		const static Color Moccasin;		// (255,228,181)
		const static Color Navajowhite;		// (255,222,173)
		const static Color Peachpuff;		// (255,218,185)
		const static Color Mistyrose;		// (255,228,225)
		const static Color Lavenderblush;	// (255,240,245)
		const static Color Seashell;		// (255,245,238)
		const static Color Oldlace;			// (253,245,230)
		const static Color Ivory;			// (255,255,240)
		const static Color Honeydew;		// (240,255,240)
		const static Color Mintcream;		// (245,255,250)
		const static Color Azure;			// (240,255,255)
		const static Color Aliceblue;		// (240,248,255)
		const static Color Lavender;		// (230,230,250)
		const static Color Lightsteelblue;	// (176,196,222)
		const static Color Lightslategray;	// (119,136,153)
		const static Color Slategray;		// (112,128,144)
		const static Color Steelblue;		// (70,130,180)
		const static Color Royalblue;		// (65,105,225)
		const static Color Midnightblue;	// (25,25,112)
		const static Color Navy;			// (0,0,128)
		const static Color Darkblue;		// (0,0,139)
		const static Color Mediumblue;		// (0,0,205)
		const static Color Blue;			// (0,0,255)
		const static Color Dodgerblue;		// (30,144,255)
		const static Color Cornflowerblue;	// (100,149,237)
		const static Color Deepskyblue;		// (0,191,255)
		const static Color Lightskyblue;	// (135,206,250)
		const static Color Skyblue;			// (135,206,235)
		const static Color Lightblue;		// (173,216,230)
		const static Color Powderblue;		// (176,224,230)
		const static Color Paleturquoise;	// (175,238,238)
		const static Color Lightcyan;		// (224,255,255)
		const static Color Cyan;			// (0,255,255)
		const static Color Aqua;			// (0,255,255)
		const static Color Turquoise;		// (64,224,208)
		const static Color Mediumturquoise;	// (72,209,204)
		const static Color Darkturquoise;	// (0,206,209)
		const static Color Lightseagreen;	// (32,178,170)
		const static Color Cadetblue;		// (95,158,160)
		const static Color Darkcyan;		// (0,139,139)
		const static Color Teal;			// (0,128,128)
		const static Color Darkslategray;	// (47,79,79)
		const static Color Darkgreen;		// (0,100,0)
		const static Color Green;			// (0,128,0)
		const static Color Forestgreen;		// (34,139,34)
		const static Color Seagreen;		// (46,139,87)
		const static Color Mediumseagreen;	// (60,179,113)
		const static Color Mediumaquamarine;// (102,205,170)
		const static Color Darkseagreen;	// (143,188,143)
		const static Color Aquamarine;		// (127,255,212)
		const static Color Palegreen;		// (152,251,152)
		const static Color Lightgreen;		// (144,238,144)
		const static Color Springgreen;		// (0,255,127)
		const static Color Mediumspringgreen;// (0,250,154)
		const static Color Lawngreen;		// (124,252,0)
		const static Color Chartreuse;		// (127,255,0)
		const static Color Greenyellow;		// (173,255,47)
		const static Color Lime;			// (0,255,0)
		const static Color Limegreen;		// (50,205,50)
		const static Color Yellowgreen;		// (154,205,50)
		const static Color Darkolivegreen;	// (85,107,47)
		const static Color Olivedrab;		// (107,142,35)
		const static Color Olive;			// (128,128,0)
		const static Color Darkkhaki;		// (189,183,107)
		const static Color Palegoldenrod;	// (238,232,170)
		const static Color Cornsilk;		// (255,248,220)
		const static Color Beige;			// (245,245,220)
		const static Color Lightyellow;		// (255,255,224)
		const static Color Lightgoldenrodyellow;// (250,250,210)
		const static Color Lemonchiffon;	// (255,250,205)
		const static Color Wheat;			// (245,222,179)
		const static Color Burlywood;		// (222,184,135)
		const static Color Tan;				// (210,180,140)
		const static Color Khaki;			// (240,230,140)
		const static Color Yellow;			// (255,255,0)
		const static Color Gold;			// (255,215,0)
		const static Color Orange;			// (255,165,0)
		const static Color Sandybrown;		// (244,164,96)
		const static Color Darkorange;		// (255,140,0)
		const static Color Goldenrod;		// (218,165,32)
		const static Color Peru;			// (205,133,63)
		const static Color Darkgoldenrod;	// (184,134,11)
		const static Color Chocolate;		// (210,105,30)
		const static Color Sienna;			// (160,82,45)
		const static Color Saddlebrown;		// (139,69,19)
		const static Color Maroon;			// (128,0,0)
		const static Color Darkred;			// (139,0,0)
		const static Color Brown;			// (165,42,42)
		const static Color Firebrick;		// (178,34,34)
		const static Color Indianred;		// (205,92,92)
		const static Color Rosybrown;		// (188,143,143)
		const static Color Darksalmon;		// (233,150,122)
		const static Color Lightcoral;		// (240,128,128)
		const static Color Salmon;			// (250,128,114)
		const static Color Lightsalmon;		// (255,160,122)
		const static Color Coral;			// (255,127,80)
		const static Color Tomato;			// (255,99,71)
		const static Color Orangered;		// (255,69,0)
		const static Color Red;				// (255,0,0)
		const static Color Crimson;			// (220,20,60)
		const static Color Mediumvioletred;	// (199,21,133)
		const static Color Deeppink;		// (255,20,147)
		const static Color Hotpink;			// (255,105,180)
		const static Color Palevioletred;	// (219,112,147)
		const static Color Pink;			// (255,192,203)
		const static Color Lightpink;		// (255,182,193)
		const static Color Thistle;			// (216,191,216)
		const static Color Magenta;			// (255,0,255)
		const static Color Fuchsia;			// (255,0,255)
		const static Color Violet;			// (238,130,238)
		const static Color Plum;			// (221,160,221)
		const static Color Orchid;			// (218,112,214)
		const static Color Mediumorchid;	// (186,85,211)
		const static Color Darkorchid;		// (153,50,204)
		const static Color Darkviolet;		// (148,0,211)
		const static Color Darkmagenta;		// (139,0,139)
		const static Color Purple;			// (128,0,128)
		const static Color Indigo;			// (75,0,130)
		const static Color Darkslateblue;	// (72,61,139)
		const static Color Blueviolet;		// (138,43,226)
		const static Color Mediumpurple;	// (147,112,219)
		const static Color Slateblue;		// (106,90,205)
		const static Color Mediumslateblue;	// (123,104,238)
	};

	/// <summary>
	/// カラー
	/// </summary>
	/// <remarks>
	/// R,G,B,A それぞれ 8bit 整数の色情報を持ちます。
	/// </remarks>
	union Color
	{
		unsigned c;
		unsigned char elem[4];
# pragma warning ( disable: 4201 )	
		struct
		{
			unsigned r : 8;
			unsigned g : 8;
			unsigned b : 8;
			unsigned a : 8;
		};
# pragma warning ( default: 4201 )

		/**
		*
		*/
		Color() {}

		/**
		*
		*	@param _r
		*	@param _g
		*	@param _b
		*	@param _a
		*/
		Color(unsigned _r, unsigned _g, unsigned _b, unsigned _a = 255)
			: r(_r & 0xFF)
			, g(_g & 0xFF)
			, b(_b & 0xFF)
			, a(_a & 0xFF) {}

		/**
		*
		*	@param rgb
		*/
		explicit Color(unsigned rgb)
			: r(rgb & 0xFF)
			, g(rgb & 0xFF)
			, b(rgb & 0xFF)
			, a(0xFF) {}

		/**
		*
		*	@param _rgb
		*	@param _a
		*/
		Color(unsigned _rgb, unsigned _a) 
			: r(_rgb & 0xFF)
			, g(_rgb & 0xFF)
			, b(_rgb & 0xFF)
			, a(_a & 0xFF) {}

		/**
		*
		*	@param col
		*	@param _a
		*/
		Color(const Color& col, unsigned _a)
			: c(col.c)
		{
			a = _a;
		}

		/**
		*
		*	@param col
		*/
		Color(const Color& col)
			: c(col.c) {}

		/**
		*
		*	@param col
		*/
		Color(const ColorF& col);

		/**
		*	カラーを初期化します。
		*	@param code L"#FFF" または L"#FFFFFF" のような # から始まるカラーコード
		*/
		Color(const StringView& code);

		/**
		*
		*	@param col
		*	@return
		*/
		Color& operator = (const Color& col);

		/**
		*
		*	@param col
		*	@return
		*/
		Color& operator = (const ColorF& col);

		/**
		*
		*	@param rgb
		*	@return
		*/
		Color& operator = (unsigned rgb);

		/**
		*
		*	@return
		*/
		Color operator ~ () const;

		/**
		*
		*	@param col
		*	@return
		*/
		bool operator == (const Color& col) const;

		/**
		*
		*	@param col
		*	@return
		*/
		bool operator != (const Color& col) const;

		/**
		*
		*	@param _r
		*	@param _g
		*	@param _b
		*	@param _a
		*	@return
		*/
		Color& set(unsigned _r, unsigned _g, unsigned _b, unsigned _a = 255);

		/**
		*
		*	@param rgb
		*	@return
		*/
		Color& setRGB(unsigned rgb);

		/**
		*
		*	@param alpha
		*	@return
		*/
		Color& setAlpha(unsigned alpha);

		/**
		*
		*/
		Property_Get(unsigned, code) const;

		/// <summary>
		/// グレイスケール値 [0, 255]
		/// </summary>
		Property_Get(unsigned char, grayscale) const;

		/// <summary>
		/// グレイスケール値 [0.0, 1.0]
		/// </summary>
		Property_Get(double, grayscaleF) const;
	};

	/**
	*
	*	@param os
	*	@param color
	*	@return
	*/
	template<typename CharType>
	inline
	std::basic_ostream<CharType>&
		operator << (
		std::basic_ostream<CharType>& os,
		const Color& color
		)
	{
		return os << CharType('(')
			<< color.r << CharType(',')
			<< color.g << CharType(',')
			<< color.b << CharType(',')
			<< color.a << CharType(')');
	}

	/**
	*
	*	@param is
	*	@param color
	*	@return
	*/
	template<typename CharType>
	inline
	std::basic_istream<CharType>&
		operator >> (
		std::basic_istream<CharType>& is,
		Color& color
		)
	{
		unsigned cols[4];
		CharType unused;

		is >> unused
			>> cols[0] >> unused
			>> cols[1] >> unused
			>> cols[2] >> unused;

		if (unused == CharType(','))
		{
			is >> cols[3] >> unused;
		}
		else
		{
			cols[3] = 255;
		}

		for (int i = 0; i < 4; ++i)
		{
			color.elem[i] = cols[i] & 0xFF;
		}

		return is;
	}
}

namespace s3d
{
	/// <summary>
	/// カラー
	/// </summary>
	/// <remarks>
	/// R,G,B,A それぞれ 64bit 浮動小数の色情報を持ちます。
	/// </remarks>
	struct ColorF
	{
# pragma warning ( disable: 4201 )
		union
		{
			double elem[4];
			struct
			{
				double r, g, b, a;
			};
		};
# pragma warning ( default: 4201 )

		/**
		*
		*/
		ColorF() {}

		/**
		*
		*	@param _rgb
		*/
		explicit ColorF(double _rgb)
			: r(_rgb)
			, g(_rgb)
			, b(_rgb)
			, a(1.0) {}

		/**
		*
		*	@param _rgb
		*	@param _a
		*/
		ColorF(double _rgb, double _a)
			: r(_rgb)
			, g(_rgb)
			, b(_rgb)
			, a(_a) {}

		/**
		*
		*	@param _r
		*	@param _g
		*	@param _b
		*	@param _a
		*/
		ColorF(double _r, double _g, double _b, double _a = 1.0)
			: r(_r)
			, g(_g)
			, b(_b)
			, a(_a) {}

		/**
		*
		*	@param col
		*	@param _a
		*/
		ColorF(const ColorF& col, double _a)
			: r(col.r)
			, g(col.g)
			, b(col.b)
			, a(_a) {}
		
		/**
		*
		*	@param rgb
		*/
		explicit ColorF(const Vec3& rgb);

		/**
		*
		*	@param rgb
		*	@param _a
		*/
		ColorF(const Vec3& rgb, double _a);

		/**
		*
		*	@param col
		*/
		ColorF(const Color& col)
			: r(col.r / 255.0)
			, g(col.g / 255.0)
			, b(col.b / 255.0)
			, a(col.a / 255.0) {}

		/**
		*
		*	@param col
		*/
		ColorF(const ColorF& col)
			: r(col.r)
			, g(col.g)
			, b(col.b)
			, a(col.a) {}

		/**
		*	カラーを初期化します。
		*	@param code L"#FFF" または L"#FFFFFF" のような # から始まるカラーコード
		*/
		ColorF(const StringView& code);

		/**
		*
		*	@param col
		*	@return
		*/
		ColorF& operator = (const ColorF& col);

		/**
		*
		*	@param col
		*	@return
		*/
		ColorF& operator = (const Color& col);

		/**
		*
		*	@param rgb
		*	@return
		*/
		ColorF& operator = (double rgb);

		/**
		*
		*	@param s
		*	@return
		*/
		ColorF operator * (double s) const;

		/**
		*
		*	@param col
		*	@return
		*/
		ColorF& operator += (const ColorF& col);

		/**
		*
		*	@param col
		*	@return
		*/
		ColorF& operator -= (const ColorF& col);

		/**
		*
		*	@param s
		*	@return
		*/
		ColorF& operator *= (double s);

		/**
		*
		*	@param col
		*	@return
		*/
		ColorF& operator *= (const ColorF& col);

		/**
		*
		*	@param _r
		*	@param _g
		*	@param _b
		*	@param _a
		*	@return
		*/
		ColorF& set(double _r, double _g, double _b, double _a = 1.0);

		/**
		*
		*	@param alpha
		*	@return
		*/
		ColorF& setRGB(double rgb);

		/**
		*
		*	@param alpha
		*	@return
		*/
		ColorF& setAlpha(double alpha);

		/**
		*
		*/
		Color toColor() const;

		ColorF gamma(double gamma) const;

		/// <summary>
		/// グレイスケール値 [0.0, 1.0]
		/// </summary>
		Property_Get(double, grayscale) const;

		/**
		*
		*/
		Property_Get(Vec2, rg) const;

		/**
		*
		*/
		Property_Get(Vec2, gb) const;

		/**
		*
		*/
		Property_Get(Vec2, ba) const;

		/**
		*
		*/
		Property_Get(Vec3, rgb) const;

		/**
		*
		*/
		Property_Get(Vec3, gba) const;

		/**
		*
		*/
		Property_Get(Vec4, rgba) const;

		/**
		*
		*/
		Property_Get(Vec4, argb) const;
	};

	/**
	*
	*	@param os
	*	@param color
	*	@return
	*/
	template<typename CharType>
	inline
	std::basic_ostream<CharType>&
		operator << (
		std::basic_ostream<CharType>& os,
		const ColorF& color
		)
	{
		return os << CharType('(')
			<< color.r << CharType(',')
			<< color.g << CharType(',')
			<< color.b << CharType(',')
			<< color.a << CharType(')');
	}

	/**
	*
	*	@param is
	*	@param color
	*	@return
	*/
	template<typename CharType>
	inline
	std::basic_istream<CharType>&
		operator >> (
		std::basic_istream<CharType>& is,
		ColorF& color
		)
	{
		CharType unused;

		is >> unused
			>> color.r >> unused
			>> color.g >> unused
			>> color.b >> unused;

		if (unused == CharType(','))
		{
			is >> color.a >> unused;
		}
		else
		{
			color.a = 1.0;
		}

		return is;
	}

	/// <summary>
	/// Color(255, 255, 255, alpha) を返します。
	/// </summary>
	/// <param name="alpha">
	/// アルファ値
	/// </param>
	/// <returns>
	/// Color(255, 255, 255, alpha)
	/// </returns>
	inline Color Alpha(unsigned alpha)
	{
		return{ 255, alpha };
	}

	/// <summary>
	/// ColorF(1.0, 1.0, 1.0, alpha) を返します。
	/// </summary>
	/// <param name="alpha">
	/// アルファ値
	/// </param>
	/// <returns>
	/// ColorF(1.0, 1.0, 1.0, alpha)
	/// </returns>
	inline ColorF AlphaF(double alpha)
	{
		return{ 1.0, alpha };
	}

	/**
	*
	*	@param dst
	*	@param src
	*	@param alpha
	*	@return
	*/
	inline Color BlendColor(const Color& dst, const Color& src, unsigned alpha = 255)
	{
		const unsigned a = (src.a * alpha) / 255;
		const unsigned r = ((src.r * a) + (dst.r * (255 - a))) / 255;
		const unsigned g = ((src.g * a) + (dst.g * (255 - a))) / 255;
		const unsigned b = ((src.b * a) + (dst.b * (255 - a))) / 255;

		return{ r, g, b, dst.a };
	}

	/**
	*
	*	@param dst
	*	@param src
	*	@param alpha
	*	@return
	*/
	Color BlendColor(const ColorF& dst, const ColorF& src, double alpha = 1.0);

	/**
	*
	*	@param s
	*	@param col
	*	@return
	*/
	const ColorF operator * (double s, const ColorF& col);

	/**
	*
	*	@param col1
	*	@param col2
	*	@return
	*/
	const ColorF operator * (const ColorF& col1, const ColorF& col2);
}

# include "SivColor.inl"
