﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

namespace s3d
{
	inline Color::Color(const ColorF& col)
		: Color(col.toColor())
	{

	}

	inline Color& Color::operator = (const Color& col)
	{
		c = col.c;
		return *this;
	}

	inline Color& Color::operator = (const ColorF& col)
	{
		return *this = col.toColor();
	}

	inline Color& Color::operator = (unsigned rgb)
	{
		r = g = b = (rgb & 0xFF);
		a = 255;
		return *this;
	}

	inline Color Color::operator ~ () const
	{
		Color col;
		col.c = ~c;
		col.a = a;
		return col;
	}

	inline bool Color::operator == (const Color& col) const
	{
		return code == col.code;
	}

	inline bool Color::operator != (const Color& col) const
	{
		return code != col.code;
	}

	inline Color& Color::set(unsigned _r, unsigned _g, unsigned _b, unsigned _a)
	{
		r = _r;
		g = _g;
		b = _b;
		a = _a;
		return *this;
	}

	inline Color& Color::setRGB(unsigned rgb)
	{
		r = g = b = rgb;
		return *this;
	}

	inline Color& Color::setAlpha(unsigned alpha)
	{
		a = alpha;
		return *this;
	}

	inline unsigned Color::_get_code() const
	{
		return c;
	}

	inline unsigned char Color::_get_grayscale() const
	{
		return static_cast<unsigned char>(0.299*r + 0.587*g + 0.114*b);
	}

	inline double Color::_get_grayscaleF() const
	{
		return (0.299/255.0*r + 0.587/255.0*g + 0.114/255.0*b);
	}

	inline ColorF& ColorF::operator = (const ColorF& col)
	{
		r = col.r;
		g = col.g;
		b = col.b;
		a = col.a;
		return *this;
	}

	inline ColorF& ColorF::operator = (const Color& col)
	{
		r = col.r / 255.0;
		g = col.g / 255.0;
		b = col.b / 255.0;
		a = col.a / 255.0;
		return *this;
	}

	inline ColorF& ColorF::operator = (double rgb)
	{
		r = g = b = rgb;
		a = 1.0;
		return *this;
	}

	inline ColorF ColorF::operator * (double s) const
	{
		return{ r*s, g*s, b*s, a };
	}

	inline ColorF& ColorF::operator += (const ColorF& col)
	{
		r += col.r;
		g += col.g;
		b += col.b;
		return *this;
	}

	inline ColorF& ColorF::operator -= (const ColorF& col)
	{
		r -= col.r;
		g -= col.g;
		b -= col.b;
		return *this;
	}

	inline ColorF& ColorF::operator *= (double s)
	{
		r *= s;
		g *= s;
		b *= s;
		return *this;
	}

	inline ColorF& ColorF::operator *= (const ColorF& col)
	{
		r *= col.r;
		g *= col.g;
		b *= col.b;
		return *this;
	}

	inline ColorF& ColorF::set(double _r, double _g, double _b, double _a)
	{
		r = _r;
		g = _g;
		b = _b;
		a = _a;
		return *this;
	}

	inline ColorF& ColorF::setRGB(double rgb)
	{
		r = g = b = rgb;
		return *this;
	}

	inline ColorF& ColorF::setAlpha(double alpha)
	{
		a = alpha;
		return *this;
	}

	inline const ColorF operator * (double s, const ColorF& col)
	{
		ColorF c = col;
		return c *= s;
	}

	inline const ColorF operator * (const ColorF& col1, const ColorF& col2)
	{
		return{ col1.r*col2.r, col1.g*col2.g, col1.b*col2.b, col1.a };
	}
}
