﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "Siv3DFwd.hpp"
# include "SivOptional.hpp"
# include "SivPointVector.hpp"
# include "SivQuaternion.hpp"
# include "SivMat4x4.hpp"

namespace s3d
{
	/// <summary>
	/// 円錐
	/// </summary>
	struct Cone
	{
		Quaternion rotation = Quaternion::Identity();

#pragma warning(disable:4201)
		union
		{
			struct { Vec3 center; };
			struct { double x, y, z; };
		};

#pragma warning(default:4201)

		double r, h;

		Cone() {}

		explicit Cone(
			double _r,
			double _h,
			const Quaternion& _rotation = Quaternion::Identity()
			)
			: Cone({ 0, 0, 0 }, _r, _h, _rotation) {}

		Cone(
			double _x,
			double _y,
			double _z,
			double _r,
			double _h,
			const Quaternion& _rotation = Quaternion::Identity()
			)
			: Cone({ _x, _y, _z }, _r, _h, _rotation) {}


		Cone(
			const Vec3& pos,
			double _r,
			double _h,
			const Quaternion& _rotation = Quaternion::Identity()
			)
			: center(pos)
			, r(_r)
			, h(_h)
			, rotation(_rotation) {}

		Cone& setPos(double _x, double _y, double _z) { return setPos({ _x, _y, _z }); }

		Cone& setPos(const Vec3& _pos) { center.set(_pos); return *this; }

		Cone& setSize(double _r, double _h) { r = _r; h = _h;  return *this; }

		Cone& moveBy(double _x, double _y, double _z) { return moveBy({ _x, _y, _z }); }

		Cone& moveBy(const Vec3& v) { center.moveBy(v); return *this; }


		Cone movedBy(double _x, double _y, double _z) const { return movedBy({ _x, _y, _z }); }

		Cone movedBy(const Vec3& v) const { return{ center.movedBy(v), r, h, rotation }; }


		Cone rollPitchYaw(double roll, double pitch, double yaw) const { return{ center, r, h, rotation.rollPitchYaw(roll, pitch, yaw) }; }

		Cone rollPitchYaw(const Quaternion& _rotation) const { return{ center, r, h, rotation * _rotation }; }

		Mat4x4 __vectorcall getMatrix() const { return Mat4x4::AffineTransform({ r, h, r }, rotation, center); }

		/**
		*
		*	@param primitive
		*	@return
		*/
		/*
		template<typename Primitive>
		bool intersects(const Primitive& primitive) const
		{
		return Geometry3D::Intersect(*this, primitive);
		}
		*/

		void draw(const ColorF& color = ColorF(1.0)) const;

		void draw(const Texture& texture, const ColorF& color = ColorF(1.0)) const;

		void drawForward(const ColorF& color = ColorF(1.0)) const;

		void drawForward(const Texture& texture, const ColorF& color = ColorF(1.0)) const;
	};
}
