﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <memory>

namespace s3d
{
	enum class ShaderStage
	{
		Vertex,

		Pixel,
	};

	class ConstantBufferBase
	{
	private:

		class ConstantBufferDetail;

		std::shared_ptr<ConstantBufferDetail> m_detail;

	public:

		ConstantBufferBase();

		bool init(int size);

		bool update(const void* data, size_t size);

		const ConstantBufferDetail* detail() const;
	};

	template<typename Type>
	class ConstantBuffer
	{
	private:

		enum { Size = sizeof(Type) };

		static_assert(Size <= 16 * 4096, "");

		ConstantBufferBase m_base;

		Type* m_data = static_cast<Type*>(::_aligned_malloc(Size, 16));

	public:

		ConstantBuffer()
		{
			m_base.init(Size);
		}

		~ConstantBuffer()
		{
			::_aligned_free(m_data);
		}

		Type& get()
		{
			return *m_data;
		}

		bool update()
		{
			return m_base.update(m_data, sizeof(Type));
		}

		bool update(const Type& data)
		{
			*m_data = data;

			return update();
		}

		ConstantBufferBase& base()
		{
			return m_base;
		}
	};
}
