﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "SivGrid.hpp"
# include "SivCustomColor.hpp"
# include "SivImage.hpp"
# include "SivTextureFormat.hpp"

namespace s3d
{
	template<typename Type>
	class CustomImage : public Grid<Type>
	{
	public:

		using ColorType = Type;

		const static TextureFormat Format = Type::Format;

		CustomImage() {}

		CustomImage(int width, int height)
			: Grid(width, height)
		{

		}

		CustomImage(int width, int height, const Type& value)
			: Grid(width, height, value)
		{

		}

		CustomImage(const FilePath& path);

		/**
		*	イメージの高さ（ピクセル）
		*/
		Property_Get(Point, size) const { return{ width, height }; }

		/**
		*
		*/
		Property_Get(unsigned, stride) const { return width * sizeof(Type); }

		/**
		*
		*/
		Property_Get(unsigned, num_pixels) const { return width * height; }

		/**
		*	イメージのデータサイズ（バイト）
		*/
		Property_Get(unsigned, dataSize) const { return stride * height; }

		Image toImage() const
		{
			Image image(width, height);

			auto itDst = image.begin();

			const auto itEnd = end();

			for (auto it = begin(); it != itEnd; ++it)
			{
				*itDst = it->toColor();

				++itDst;
			}

			return image;
		}

		void fromImage(const Image& image)
		{
			resize(image.width, image.height);

			auto itDst = begin();

			const auto itEnd = image.end();

			for (auto it = image.begin(); it != itEnd; ++it)
			{
				itDst->fromColor(*it);

				++itDst;
			}
		}

		bool
			saveDDS(
			const FilePath& path
			) const;

		MemoryReader
			encodeDDS(
			) const;
	};

	using ImageR32F		= CustomImage<R32F>;
	using ImageRGBA16F	= CustomImage<RGBA16F>;
	using ImageRGBA32F	= CustomImage<RGBA32F>;
}
