﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <functional>
struct ID3D11DeviceContext;

namespace s3d
{
	using customRendererCallback = std::function<bool(void *userdata)>;

	namespace CustomRenderer
	{
		ID3D11DeviceContext* GetDeviceContext();

		// while(System::Update())
		// {
		//		main loop
		//		-> draw 2D-Background
		// }
		//
		// in System::Update()
		// [
		// -> Deferred Rendering G-Buffer
		// -> [DeferredCallback]
		// -> Deferred Rendering Light Accumulation
		// -> Deferred Rendering Finalize
		// -> Forward Rendereing
		// -> [ForwardCallback]
		// -> FXAA
		// -> [PostProcessCallback]
		// -> 2D Rendering
		// -> [2DCallback]
		// -> MSAA
		// -> present
		// -> clear
		// ]

		// RT0: G-Buffer.Albedo, RT1: G-Buffer.Depth, RT2: G-Buffer.Normal, DS: 3D
		void SetDeferredCallback(customRendererCallback callback, void* userData = nullptr);

		// RT0: 3D, RT1: (none), RT2: (none), DS: 3D
		void SetForwardCallback(customRendererCallback callback, void* userData = nullptr);

		// RT0: 2D, RT1: (none), RT2: (none), DS: (none)
		void SetPostProcessCallback(customRendererCallback callback, void* userData = nullptr);

		// RT0: 2D, RT1: (none), RT2: (none), DS: 2D
		void Set2DCallback(customRendererCallback callback, void* userData = nullptr);
	}
}
