﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <chrono>

namespace s3d
{
	/// <summary>
	/// 日数
	/// </summary>
	using Days = std::chrono::duration<int, std::ratio<86400>>;

	/// <summary>
	/// 日数
	/// </summary>
	using DaysF = std::chrono::duration<long double, std::ratio<86400>>;


	/// <summary>
	/// 時間
	/// </summary>
	using Hours = std::chrono::hours;

	/// <summary>
	/// 時間
	/// </summary>
	using HoursF = std::chrono::duration<long double, std::ratio<3600>>;


	/// <summary>
	/// 分
	/// </summary>
	using Minutes = std::chrono::minutes;

	/// <summary>
	/// 分
	/// </summary>
	using MinutesF = std::chrono::duration<long double, std::ratio<60>>;


	/// <summary>
	/// 秒
	/// </summary>
	using Seconds = std::chrono::seconds;

	/// <summary>
	/// 秒
	/// </summary>
	using SecondsF = std::chrono::duration<long double>;


	/// <summary>
	/// ミリ秒
	/// </summary>
	using Milliseconds = std::chrono::milliseconds;

	/// <summary>
	/// ミリ秒
	/// </summary>
	using MillisecondsF = std::chrono::duration<long double, std::milli>;


	/// <summary>
	/// マイクロ秒
	/// </summary>
	using Microseconds = std::chrono::microseconds;

	/// <summary>
	/// マイクロ秒
	/// </summary>
	using MicrosecondsF = std::chrono::duration<long double, std::micro>;


	/// <summary>
	/// ナノ秒
	/// </summary>
	using Nanoseconds = std::chrono::nanoseconds;

	/// <summary>
	/// ナノ秒
	/// </summary>
	using NanosecondsF = std::chrono::duration<long double, std::nano>;

	/// <summary>
	/// 時間の単位を変換します。
	/// </summary>
	/// <param name="duration">
	/// 変換する時間
	/// </param>
	/// <returns>
	/// DurationTo 型に変換された時間
	/// </returns>
	template<typename DurationTo, typename DurationFrom> inline DurationTo DurationCast(const DurationFrom& duration)
	{
		return std::chrono::duration_cast<DurationTo>(duration);
	}
}
