﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "Siv3DFwd.hpp"
# include "SivKey.hpp"
# include "SivTouch.hpp"
# include "SivGamepad.hpp"

namespace s3d
{
	/// <summary>
	/// 入力
	/// </summary>
	/// <remarks>
	/// ユーザの入力に関する機能を提供します。
	/// </remarks>
	struct Input
	{
		/// <summary>
		/// マウス左ボタン
		/// </summary>
		static Key MouseL;

		/// <summary>
		/// マウス右ボタン
		/// </summary>
		static Key MouseR;

		/// <summary>
		/// マウス中央ボタン
		/// </summary>
		static Key MouseM;

		/// <summary>
		/// マウス X1 ボタン
		/// </summary>
		static Key MouseX1;

		/// <summary>
		/// マウス X2 ボタン
		/// </summary>
		static Key MouseX2;

		/// <summary>
		/// Ctrl + break キー
		/// </summary>
		static Key KeyCancel;

		/// <summary>
		/// Backspace キー
		/// </summary>
		static Key KeyBackspace;

		/// <summary>
		/// Tab キー
		/// </summary>
		static Key KeyTab;

		/// <summary>
		/// Clear(NumLock を外した状態のテンキー5) キー
		/// </summary>
		static Key KeyClear;

		/// <summary>
		/// Enter キー
		/// </summary>
		static Key KeyEnter;

		/// <summary>
		/// Shift キー
		/// </summary>
		static Key KeyShift;

		/// <summary>
		/// Ctrl キー
		/// </summary>
		static Key KeyControl;

		/// <summary>
		/// Alt キー
		/// </summary>
		static Key KeyAlt;

		/// <summary>
		/// Pause キー
		/// </summary>
		static Key KeyPause;

		/// <summary>
		/// Escape キー
		/// </summary>
		static Key KeyEscape;

		/// <summary>
		/// スペースキー
		/// </summary>
		static Key KeySpace;

		/// <summary>
		/// PageUp キー
		/// </summary>
		static Key KeyPageUp;

		/// <summary>
		/// PageDown キー
		/// </summary>
		static Key KeyPageDown;

		/// <summary>
		/// End キー
		/// </summary>
		static Key KeyEnd;

		/// <summary>
		/// Home キー
		/// </summary>
		static Key KeyHome;

		/// <summary>
		/// ← キー
		/// </summary>
		static Key KeyLeft;

		/// <summary>
		/// ↑ キー
		/// </summary>
		static Key KeyUp;

		/// <summary>
		/// → キー
		/// </summary>
		static Key KeyRight;

		/// <summary>
		/// ↓ キー
		/// </summary>
		static Key KeyDown;

		/// <summary>
		/// PrintScreen キー
		/// </summary>
		static Key KeyPrintScreen;

		/// <summary>
		/// Insert キー
		/// </summary>
		static Key KeyInsert;

		/// <summary>
		/// Delete キー
		/// </summary>
		static Key KeyDelete;

		/// <summary>
		/// 左 Shift キー
		/// </summary>
		static Key KeyLShift;

		/// <summary>
		/// 右 Shift キー
		/// </summary>
		static Key KeyRShift;

		/// <summary>
		/// 左 Ctrl キー
		/// </summary>
		static Key KeyLControl;

		/// <summary>
		/// 右 Ctrl キー
		/// </summary>
		static Key KeyRControl;

		/// <summary>
		/// 左 Alt キー
		/// </summary>
		static Key KeyLAlt;

		/// <summary>
		/// 右 Alt キー
		/// </summary>
		static Key KeyRAlt;

		/// <summary>
		/// 0 キー
		/// </summary>
		static Key Key0;

		/// <summary>
		/// 1 キー
		/// </summary>
		static Key Key1;

		/// <summary>
		/// 2 キー
		/// </summary>
		static Key Key2;

		/// <summary>
		/// 3 キー
		/// </summary>
		static Key Key3;

		/// <summary>
		/// 4 キー
		/// </summary>
		static Key Key4;

		/// <summary>
		/// 5 キー
		/// </summary>
		static Key Key5;

		/// <summary>
		/// 6 キー
		/// </summary>
		static Key Key6;

		/// <summary>
		/// 7 キー
		/// </summary>
		static Key Key7;

		/// <summary>
		/// 8 キー
		/// </summary>
		static Key Key8;

		/// <summary>
		/// 9 キー
		/// </summary>
		static Key Key9;

		/// <summary>
		/// A キー
		/// </summary>
		static Key KeyA;

		/// <summary>
		/// B キー
		/// </summary>
		static Key KeyB;

		/// <summary>
		/// C キー
		/// </summary>
		static Key KeyC;

		/// <summary>
		/// D キー
		/// </summary>
		static Key KeyD;

		/// <summary>
		/// E キー
		/// </summary>
		static Key KeyE;

		/// <summary>
		/// F キー
		/// </summary>
		static Key KeyF;

		/// <summary>
		/// G キー
		/// </summary>
		static Key KeyG;

		/// <summary>
		/// H キー
		/// </summary>
		static Key KeyH;

		/// <summary>
		/// I キー
		/// </summary>
		static Key KeyI;

		/// <summary>
		/// J キー
		/// </summary>
		static Key KeyJ;

		/// <summary>
		/// K キー
		/// </summary>
		static Key KeyK;

		/// <summary>
		/// L キー
		/// </summary>
		static Key KeyL;

		/// <summary>
		/// M キー
		/// </summary>
		static Key KeyM;

		/// <summary>
		/// N キー
		/// </summary>
		static Key KeyN;

		/// <summary>
		/// O キー
		/// </summary>
		static Key KeyO;

		/// <summary>
		/// P キー
		/// </summary>
		static Key KeyP;

		/// <summary>
		/// Q キー
		/// </summary>
		static Key KeyQ;

		/// <summary>
		/// R キー
		/// </summary>
		static Key KeyR;

		/// <summary>
		/// S キー
		/// </summary>
		static Key KeyS;

		/// <summary>
		/// T キー
		/// </summary>
		static Key KeyT;

		/// <summary>
		/// U キー
		/// </summary>
		static Key KeyU;

		/// <summary>
		/// V キー
		/// </summary>
		static Key KeyV;

		/// <summary>
		/// W キー
		/// </summary>
		static Key KeyW;

		/// <summary>
		/// X キー
		/// </summary>
		static Key KeyX;

		/// <summary>
		/// Y キー
		/// </summary>
		static Key KeyY;

		/// <summary>
		/// Z キー
		/// </summary>
		static Key KeyZ;

		/// <summary>
		/// Numpad 0 キー
		/// </summary>
		static Key KeyNum0;

		/// <summary>
		/// Numpad 1 キー
		/// </summary>
		static Key KeyNum1;

		/// <summary>
		/// Numpad 2 キー
		/// </summary>
		static Key KeyNum2;

		/// <summary>
		/// Numpad 3 キー
		/// </summary>
		static Key KeyNum3;

		/// <summary>
		/// Numpad 4 キー
		/// </summary>
		static Key KeyNum4;

		/// <summary>
		/// Numpad 5 キー
		/// </summary>
		static Key KeyNum5;

		/// <summary>
		/// Numpad 6 キー
		/// </summary>
		static Key KeyNum6;

		/// <summary>
		/// Numpad 7 キー
		/// </summary>
		static Key KeyNum7;

		/// <summary>
		/// Numpad 8 キー
		/// </summary>
		static Key KeyNum8;

		/// <summary>
		/// Numpad 9 キー
		/// </summary>
		static Key KeyNum9;

		/// <summary>
		/// Numpad * キー
		/// </summary>
		static Key KeyNumMultiply;

		/// <summary>
		/// Numpad + キー
		/// </summary>
		static Key KeyNumAdd;

		/// <summary>
		/// Separator キー
		/// </summary>
		static Key KeyNumSeparator;

		/// <summary>
		/// Numpad - キー
		/// </summary>
		static Key KeyNumSubtract;

		/// <summary>
		/// Numpad . キー
		/// </summary>
		static Key KeyNumDecimal;

		/// <summary>
		/// Numpad / キー
		/// </summary>
		static Key KeyNumDivide;

		/// <summary>
		/// NumLock キー
		/// </summary>
		static Key KeyNumLock;

		/// <summary>
		/// F1 キー
		/// </summary>
		static Key KeyF1;

		/// <summary>
		/// F2 キー
		/// </summary>
		static Key KeyF2;

		/// <summary>
		/// F3 キー
		/// </summary>
		static Key KeyF3;

		/// <summary>
		/// F4 キー
		/// </summary>
		static Key KeyF4;

		/// <summary>
		/// F5 キー
		/// </summary>
		static Key KeyF5;

		/// <summary>
		/// F6 キー
		/// </summary>
		static Key KeyF6;

		/// <summary>
		/// F7 キー
		/// </summary>
		static Key KeyF7;

		/// <summary>
		/// F8 キー
		/// </summary>
		static Key KeyF8;

		/// <summary>
		/// F9 キー
		/// </summary>
		static Key KeyF9;

		/// <summary>
		/// F10 キー
		/// </summary>
		static Key KeyF10;

		/// <summary>
		/// F11 キー
		/// </summary>
		static Key KeyF11;

		/// <summary>
		/// F12 キー
		/// </summary>
		static Key KeyF12;

		/// <summary>
		/// : キー
		/// </summary>
		static Key KeyColon;

		/// <summary>
		/// ; キー
		/// </summary>
		static Key KeySemicolon;

		/// <summary>
		/// , キー
		/// </summary>
		static Key KeyComma;

		/// <summary>
		/// - キー
		/// </summary>
		static Key KeyMinus;

		/// <summary>
		/// . キー
		/// </summary>
		static Key KeyPeriod;

		/// <summary>
		/// / キー
		/// </summary>
		static Key KeySlash;

		/// <summary>
		/// @ キー
		/// </summary>
		static Key KeyAt;

		/// <summary>
		/// [ キー
		/// </summary>
		static Key KeyLBracket;

		/// <summary>
		/// ] キー
		/// </summary>
		static Key KeyRBracket;

		/// <summary>
		/// \ キー
		/// </summary>
		/// </summary>
		/// <remarks>
		/// キーボードにより異なります。
		/// </remarks>
		static Key KeyYen;

		/// <summary>
		/// ^ キー
		/// </summary>
		static Key KeyCaret;

		/// <summary>
		/// バックスラッシュキー
		/// </summary>
		/// <remarks>
		/// キーボードにより異なります。
		/// </remarks>
		static Key KeyBackslash;

		/// <summary>
		/// メディア 	Next Track ボタン
		/// </summary>
		static Key KeyNextTrack;

		/// <summary>
		/// メディア 	Previous Track ボタン
		/// </summary>
		static Key KeyPreviousTrack;

		/// <summary>
		/// メディア Stop ボタン
		/// </summary>
		static Key KeyStopMedia;

		/// <summary>
		/// メディア Play / Pause ボタン
		/// </summary>
		static Key KeyPlayPauseMedia;

		/// <summary>
		/// 何らかのキーがクリックされたかを返します。
		/// </summary>
		/// <returns>
		/// 何らかのキーがクリックされた場合 true, それ以外の場合は false
		/// </returns>
		static bool AnyKeyClicked();

		/// <summary>
		/// 何らかのキーが押されているかを返します。
		/// </summary>
		/// <returns>
		/// 何らかのキーが押されている場合 true, それ以外の場合は false
		/// </returns>
		static bool AnyKeyPressed();

		/// <summary>
		/// 何らかのキーが離されたかを返します。
		/// </summary>
		/// <returns>
		/// 何らかのキーが離された場合 true, それ以外の場合は false
		/// </returns>
		static bool AnyKeyReleased();

		/// <summary>
		/// 現在アクティブなキーを取得します。
		/// </summary>
		/// <returns>
		/// 現在アクティブなキーの一覧
		/// </returns>
		static Array<Key> GetActiveKeys();

		/// <summary>
		/// 文字入力を取得します。
		/// </summary>
		/// <returns>
		/// 文字入力
		/// </returns>
		static String GetChars();

		/// <summary>
		/// 文字入力を取得し処理します。
		/// </summary>
		/// <param name="text">
		/// 文字入力処理用のバッファ
		/// </param>
		/// <returns>
		/// 文字入力
		/// </returns>
		static void GetCharsHelper(String& text);

		/// <summary>
		/// マルチタッチをサポートしているかを返します。
		/// </summary>
		/// <returns>
		/// マルチタッチをサポートしている場合 true, それ以外の場合は false
		/// </returns>
		static bool SupportsMultiTouch();

		/// <summary>
		/// 現在のマルチタッチを取得します。
		/// </summary>
		/// <returns>
		/// 現在のタッチの情報一覧
		/// </returns>
		static Array<Touch> GetTouches();

		/// <summary>
		/// PC に接続されているゲームパッドの一覧を取得します。
		/// </summary>
		/// <returns>
		/// PC に接続されているゲームパッドの一覧
		/// </returns>
		static Array<GamepadInfo> EnumerateJoypads();
	};
}
