﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <memory>
# include "Siv3DFwd.hpp"
# include "SivHandle.hpp"
# include "SivString.hpp"
# include "SivTextureFormat.hpp"

namespace s3d
{
	enum class InputClassification
	{
		PerVertex = 0,

		PerInstance = 1,
	};

	struct InputElementDesc
	{
		InputElementDesc(
			const std::string& _semanticName = "",
			unsigned _semanticIndex = 0,
			TextureFormat _format = TextureFormat::Unknown,
			unsigned _inputSlot = 0,
			unsigned _alignedByteOffset = 0,
			InputClassification _inputSlotClass = InputClassification::PerVertex,
			unsigned _instanceDataStepRate = 0
			)
			: semanticName(_semanticName)
			, semanticIndex(_semanticIndex)
			, format(_format)
			, inputSlot(_inputSlot)
			, alignedByteOffset(_alignedByteOffset)
			, inputSlotClass(_inputSlotClass)
			, instanceDataStepRate(_instanceDataStepRate) {}

		std::string semanticName;
		
		unsigned semanticIndex;
		
		TextureFormat format;
		
		unsigned inputSlot;
		
		unsigned alignedByteOffset;
		
		InputClassification inputSlotClass;
		
		unsigned instanceDataStepRate;
	};

	/**
	*	@brief
	*/
	class InputLayout
	{
	private:

		class Handle{};

		using InputLayoutHandle = Siv3DHandle<Handle>;

		std::shared_ptr<InputLayoutHandle> m_handle;

	public:

		/**
		*
		*/
		InputLayout();

		/**
		*
		*	@param path
		*	@param loopProperty
		*/
		InputLayout(const s3d::Array<s3d::InputElementDesc>& inputElementDesc, const s3d::VertexShader& vertexShader);

		/**
		*
		*/
		~InputLayout();

		/**
		*
		*/
		void release();

		/**
		*
		*	@return
		*/
		explicit operator bool() const { return !isEmpty; }

		/**
		*
		*/
		Property_Get(HandleIDType, id) const;

		/**
		*
		*/
		Property_Get(bool, isEmpty) const;

		/**
		*
		*	@param sound
		*	@return
		*/
		bool operator == (const InputLayout& inputLayout) const;

		/**
		*
		*	@param sound
		*	@return
		*/
		bool operator != (const InputLayout& inputLayout) const;
	};
}
