﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <intrin.h>
# include "SivPropertyMacro.hpp"
# include "SivTime.hpp"
# include "SivLogger.hpp"

namespace s3d
{
	/// <summary>
	/// ミリ秒カウンター
	/// </summary>
	/// <remarks>
	/// 経過時間をミリ秒で計測します。
	/// </remarks>
	class MillisecClock
	{
	private:

		unsigned long long m_start = Time::GetMicrosec();

	public:

		/// <summary>
		/// 経過時間（ミリ秒）
		/// </summary>
		Property_Get(unsigned long long, elapsed) const { return (Time::GetMicrosec() - m_start) / 1000ULL; }

		/// <summary>
		/// ログに経過時間を出力します。
		/// </summary>
		/// <returns>
		/// なし
		/// </returns>
		void log() const { LOG(elapsed, L" ms"); }
	};

	/// <summary>
	/// マイクロ秒カウンター
	/// </summary>
	/// <remarks>
	/// 経過時間をマイクロ秒で計測します。
	/// </remarks>
	class MicrosecClock
	{
	private:

		unsigned long long m_start = Time::GetMicrosec();

	public:

		/// <summary>
		/// 経過時間（マイクロ秒）
		/// </summary>
		Property_Get(unsigned long long, elapsed) const { return Time::GetMicrosec() - m_start; }

		/// <summary>
		/// ログに経過時間を出力します。
		/// </summary>
		/// <returns>
		/// なし
		/// </returns>
		void log() const { LOG(elapsed, L" μs"); }
	};

	/// <summary>
	/// CPU サイクルカウンター
	/// </summary>
	/// <remarks>
	/// 経過時間を CPU サイクル数で計測します。
	/// </remarks>
	class RDTSCClock
	{
	private:

		unsigned long long m_start = ::__rdtsc();

	public:

		/// <summary>
		/// 経過時間（CPU サイクル数）
		/// </summary>
		Property_Get(unsigned long long, elapsed) const { return ::__rdtsc() - m_start; }

		/// <summary>
		/// ログに経過時間を出力します。
		/// </summary>
		/// <returns>
		/// なし
		/// </returns>
		void log() const { LOG(elapsed, L" cycles"); }
	};
}
