﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <utility>

namespace s3d
{
	enum class FillMode : unsigned char
	{
		Wireframe	= 2,
		Solid		= 3,
	};

	enum class CullMode : unsigned char
	{
		None		= 1,
		Front		= 2,
		Back		= 3,
	};

	struct RasterizerState
	{
		RasterizerState(
			FillMode _fillMode = FillMode::Solid,
			CullMode _cullMode = CullMode::Back,
			bool _frontCounterClockwise = false,
			int _depthBias = 0
			)
			: fillMode(_fillMode)
			, cullMode(_cullMode)
			, frontCounterClockwise(_frontCounterClockwise)
			, depthBias(_depthBias) {}

		using DataType = unsigned __int64;

#pragma warning(disable:4201)
		union
		{
			struct
			{
				FillMode fillMode : 2;

				CullMode cullMode : 2;

				bool frontCounterClockwise : 4;

				int depthBias;
			};

			DataType _data;
		};
#pragma warning(default:4201)

		bool operator == (const RasterizerState& r) const
		{
			return _data == r._data;
		}

		bool operator != (const RasterizerState& r) const
		{
			return _data != r._data;
		}

		enum class Predefined
		{
			SolidBack,
			SolidFront,
			SolidNone,
			WireframeBack,
			WireframeFront,
			WireframeNone,
			Default2D = SolidNone,
			Default3D = SolidBack,
		};

		RasterizerState(Predefined predefined);

		const static Predefined SolidBack = Predefined::SolidBack;

		const static Predefined SolidFront = Predefined::SolidFront;

		const static Predefined SolidNone = Predefined::SolidNone;

		const static Predefined WireframeBack = Predefined::WireframeBack;

		const static Predefined WireframeFront = Predefined::WireframeFront;

		const static Predefined WireframeNone = Predefined::WireframeNone;

		const static Predefined Default2D = Predefined::Default2D;

		const static Predefined Default3D = Predefined::Default3D;
	};

	static_assert(sizeof(RasterizerState) == sizeof(RasterizerState::DataType), "");
}

template<>
struct std::hash<s3d::RasterizerState>
{
	size_t operator()(const s3d::RasterizerState& keyVal) const
	{
		return std::hash<s3d::RasterizerState::DataType>()(keyVal._data);
	}
};
