﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "SivPointVector.hpp"
# include "SivBlendState.hpp"
# include "SivRasterizerState.hpp"
# include "SivDepthStencilState.hpp"
# include "SivSamplerState.hpp"

namespace s3d
{
	struct RenderState
	{
		RenderState() {}

		RenderState(
			const Vec2& _scale,
			const Vec2& _offset,
			const BlendState& _blend,
			const RasterizerState& _rasterizer,
			const DepthStencilState& _depthStencil,
			const SamplerState(&_sampler)[2],
			unsigned char _stencilValue
			)
			: scale(_scale)
			, offset(_offset)
			, blend(_blend)
			, rasterizer(_rasterizer)
			, depthStencil(_depthStencil)
			, stencilValue(_stencilValue)
		{
			sampler[0] = _sampler[0];
			sampler[1] = _sampler[1];
		}

		Vec2 scale;

		Vec2 offset;

		BlendState blend;

		RasterizerState rasterizer;

		DepthStencilState depthStencil;

		SamplerState sampler[2];

		unsigned char stencilValue;

		enum class Predefined
		{
			Default2D,
			Default3D,
		};

		RenderState(Predefined predefined);

		const static Predefined Default2D = Predefined::Default2D;

		const static Predefined Default3D = Predefined::Default3D;
	};
}
