﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "Siv3DFwd.hpp"
# include "SivPointVector.hpp"

namespace s3d
{
	/**
	*	@brief スプライト頂点
	*/
	struct SpriteVertex
	{
		SpriteVertex() {}

		SpriteVertex(float x, float y, const Color& _color)
			: pos(x, y)
			, color(_color) {}

		SpriteVertex(float x, float y, const Color& _color, float u, float v)
			: pos(x, y)
			, tex(u, v)
			, color(_color) {}

		void set(float x, float y, float u, float v, const Color& _color)
		{
			pos.set(x, y);

			tex.set(u, v);

			color = _color;
		}

		void set(const Float2& _pos, float u, float v, const Color& _color)
		{
			pos = _pos;

			tex.set(u, v);

			color = _color;
		}

		void setPosColor(float x, float y, const Color& _color)
		{
			pos.set(x, y);

			color = _color;
		}

		void setPosColor(const Float2& _pos, const Color& _color)
		{
			pos = _pos;

			color = _color;
		}

		Float2 pos;

		Float2 tex;

		Color color;
	};

	/**
	*	@brief スプライト
	*/
	class Sprite
	{
	public:

		Array<SpriteVertex> vertices;

		Array<unsigned short> indices;

		Sprite() {}

		Sprite(
			unsigned vertexSize,
			unsigned indexSize
			)
			: vertices(vertexSize)
			, indices(indexSize)
		{

		}

		/**
		*	スプライトに適切なインデックス値が設定されているかを返します。
		*	@return
		*/
		bool isValid() const;

		/**
		*
		*/
		void draw() const;

		/**
		*
		*/
		void draw(const Color& color) const;

		/**
		*
		*/
		void draw(double x, double y) const;

		/**
		*
		*/
		void draw(const Vec2& pos) const;

		/**
		*
		*/
		void draw(double x, double y, const Color& color) const;

		/**
		*
		*/
		void draw(const Vec2& pos, const Color& color) const;

		/**
		*
		*/
		void draw(const Texture& texture) const;

		/**
		*
		*/
		void draw(const Texture& texture, const Color& color) const;

		/**
		*
		*/
		void draw(const Texture& texture, double x, double y) const;

		/**
		*
		*/
		void draw(const Texture& texture, const Vec2& pos) const;

		/**
		*
		*/
		void draw(const Texture& texture, double x, double y, const Color& color) const;

		/**
		*
		*/
		void draw(const Texture& texture, const Vec2& pos, const Color& color) const;
	};
}