﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include "Siv3DFwd.hpp"
# include "SivTexture.hpp"
# include "SivFloatShape.hpp"

namespace s3d
{
	/**
	*	@brief
	*/
	struct TextureRegion
	{
		/**
		*
		*/
		TextureRegion() {}

		/**
		*
		*	@param tex
		*/
		TextureRegion(
			const Texture& _texture
			)
			: texture(_texture)
			, uvRect(0.0f, 0.0f, 1.0f, 1.0f)
			, size(_texture.size) {}

		/**
		*
		*	@param tex
		*	@param l
		*	@param t
		*	@param r
		*	@param b
		*	@param sx
		*	@param sy
		*/
		TextureRegion(
			const Texture& _texture,
			float l,
			float t,
			float r,
			float b,
			double sx,
			double sy
			)
			: texture(_texture)
			, uvRect(l, t, r, b)
			, size(sx, sy) {}

		/**
		*
		*	@param tex
		*	@param l
		*	@param t
		*	@param r
		*	@param b
		*	@param s
		*/
		TextureRegion(
			const Texture& _texture,
			float l,
			float t,
			float r,
			float b,
			const Vec2& _size
			)
			: texture(_texture)
			, uvRect(l, t, r, b)
			, size(_size) {}

		/**
		*
		*	@param tex
		*	@param uv
		*	@param s
		*/
		TextureRegion(
			const Texture& _texture,
			const FloatRect& _uvRect,
			const Vec2& _size
			)
			: texture(_texture)
			, uvRect(_uvRect)
			, size(_size) {}

		/**
		*
		*/
		Texture texture;

		/**
		*
		*/
		FloatRect uvRect;

		/**
		*
		*/
		Vec2 size;

		/// <summary>
		/// テクスチャを描きます。
		/// </summary>
		/// <param name="diffuse">
		/// 乗算する色
		/// </param>
		/// <returns>
		/// 描画領域
		/// </returns>
		const RectF draw(const Color& diffuse = Palette::White) const;

		/// <summary>
		/// 指定した位置にテクスチャを描きます。
		/// </summary>
		/// <param name="x">
		/// 描画開始位置の X 座標
		/// </param>
		/// <param name="y">
		/// 描画開始位置の Y 座標
		/// </param>
		/// <param name="diffuse">
		/// 乗算する色
		/// </param>
		/// <returns>
		/// 描画領域
		/// </returns>
		const RectF draw(double x, double y, const Color& diffuse = Palette::White) const;

		/// <summary>
		/// 指定した位置にテクスチャを描きます。
		/// </summary>
		/// <param name="pos">
		/// 描画開始位置
		/// </param>
		/// <param name="diffuse">
		/// 乗算する色
		/// </param>
		/// <returns>
		/// 描画領域
		/// </returns>
		const RectF draw(const Vec2& pos, const Color& diffuse = Palette::White) const;

		/// <summary>
		/// 中心位置を指定してテクスチャを描きます。
		/// </summary>
		/// <param name="x">
		/// 中心位置の X 座標
		/// </param>
		/// <param name="y">
		/// 中心位置の X 座標
		/// </param>
		/// <param name="diffuse">
		/// 乗算する色
		/// </param>
		/// <returns>
		/// 描画領域
		/// </returns>
		const RectF drawAt(double x, double y, const Color& diffuse = Palette::White) const;

		/// <summary>
		/// 中心位置を指定してテクスチャを描きます。
		/// </summary>
		/// <param name="pos">
		/// 中心位置の座標
		/// </param>
		/// <param name="diffuse">
		/// 乗算する色
		/// </param>
		/// <returns>
		/// 描画領域
		/// </returns>
		const RectF drawAt(const Vec2& pos, const Color& diffuse = Palette::White) const;

		/**
		*
		*	@return
		*/
		const TextureRegion mirror() const;

		/**
		*
		*	@return
		*/
		const TextureRegion flip() const;

		/**
		*
		*	@param scaling
		*	@return
		*/
		const TextureRegion scale(double scaling) const;

		/**
		*
		*	@param xScaling
		*	@param yScaling
		*	@return
		*/
		const TextureRegion scale(double xScaling, double yScaling) const;

		/**
		*
		*	@param scaling
		*	@return
		*/
		const TextureRegion scale(const Vec2& scaling) const;

		/**
		*
		*	@param width
		*	@param height
		*	@return
		*/
		const TextureRegion resize(double width, double height) const;

		/**
		*
		*	@param size
		*	@return
		*/
		const TextureRegion resize(const Vec2& size) const;

		/**
		*
		*	@param radian
		*	@return
		*/
		const TexturedQuad rotate(double radian) const;

		/**
		*
		*	@param x
		*	@param y
		*	@param radian
		*	@return
		*/
		const TexturedQuad rotateAt(double x, double y, double radian) const;

		/**
		*
		*	@param pos
		*	@param radian
		*	@return
		*/
		const TexturedQuad rotateAt(const Vec2& pos, double radian) const;
	};
}
