﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <iostream>
# include "SivPropertyMacro.hpp"
# include "SivPoint.hpp"

namespace s3d
{
	/// <summary>
	/// 2 次元ベクトル
	/// </summary>
	template<typename Type>
	struct Vector2D
	{
		using value_type = Type;

		Type x, y;

		/**
		*
		*/
		Vector2D() {}
		
		/**
		*
		*	@param _x
		*	@param _y
		*/
		template<typename U, typename V>
		Vector2D(U _x, V _y) : x(static_cast<Type>(_x)), y(static_cast<Type>(_y)) {}

		/**
		*
		*	@param _x
		*	@param _y
		*/
		Vector2D(Type _x, Type _y) : x(_x), y(_y) {}

		/**
		*
		*	@param v
		*/
		Vector2D(const Point& v) : x(static_cast<Type>(v.x)), y(static_cast<Type>(v.y)) {}

		/**
		*
		*	@param v
		*/
		template<typename U>
		Vector2D(const Vector2D<U>& v) : x(static_cast<Type>(v.x)), y(static_cast<Type>(v.y)) {}

		/**
		*
		*	@return
		*/
		Vector2D operator + () const;

		/**
		*
		*	@return
		*/
		Vector2D operator - () const;

		/**
		*
		*	@param v
		*	@return
		*/
		Vector2D operator + (const Vector2D& v) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Vector2D operator - (const Vector2D& v) const;

		/**
		*
		*	@param s
		*	@return
		*/
		Vector2D operator * (Type s) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Vector2D operator * (const Vector2D& v) const;

		/**
		*
		*	@param s
		*	@return
		*/
		Vector2D operator / (Type s) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Vector2D& operator += (const Vector2D& v);

		/**
		*
		*	@param v
		*	@return
		*/
		Vector2D& operator -= (const Vector2D& v);

		/**
		*
		*	@param s
		*	@return
		*/
		Vector2D& operator *= (Type s);

		/**
		*
		*	@param s
		*	@return
		*/
		Vector2D& operator /= (Type s);

		/**
		*
		*	@param v
		*	@return
		*/
		bool operator == (const Vector2D& v) const;

		/**
		*
		*	@param v
		*	@return
		*/
		bool operator != (const Vector2D& v) const;

		/**
		*
		*/
		Property_Get(Vector2D, xx) const;

		/**
		*
		*/
		Property_Get(Vector2D, xy) const;

		/**
		*
		*/
		Property_Get(Vector2D, yx) const;

		/**
		*
		*/
		Property_Get(Vector2D, yy) const;

		/**
		*
		*	@return
		*/
		Point asPoint() const;

		/**
		*
		*	@param _x
		*	@param _y
		*	@return
		*/
		Vector2D& set(Type _x, Type _y) { x = _x; y = _y; return *this; }

		/**
		*
		*	@param v
		*	@return
		*/
		Vector2D& set(const Vector2D& v) { return *this = v; }

		/**
		*
		*	@param _x
		*	@param _y
		*	@return
		*/
		Vector2D movedBy(Type _x, Type _y) const { return{ x + _x, y + _y }; }

		/**
		*
		*	@param v
		*	@return
		*/
		Vector2D movedBy(const Vector2D& v) const { return{ x + v.x, y + v.y }; }

		/**
		*
		*	@param _x
		*	@param _y
		*	@return
		*/
		Vector2D& moveBy(Type _x, Type _y) { x += _x; y += _y; return *this; }

		/**
		*
		*	@param v
		*	@return
		*/
		Vector2D& moveBy(const Vector2D& v) { return *this += v; }

		/**
		*
		*	@param shape
		*	@return
		*/
		template<typename Shape>
		bool intersects(const Shape& shape) const
		{
			return Geometry2D::Intersect(*this, shape);
		}

		/**
		*
		*/
		Property_Get(bool, isZero) const;

		/**
		*
		*	@return
		*/
		Type
			length(
			) const;

		/**
		*
		*	@return
		*/
		Type
			lengthSq(
			) const;

		/**
		*
		*	@return
		*/
		Type
			lengthInv(
			) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Type
			distanceFrom(
			const Vector2D& v
			) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Type
			distanceFromSq(
			const Vector2D& v
			) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Type
			dot(
			const Vector2D& v
			) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Type
			cross(
			const Vector2D& v
			) const;

		/**
		*
		*	@return
		*/
		Vector2D
			normalized(
			) const;

		/**
		*
		*	@return
		*/
		Vector2D&
			normalize(
			);

		/**
		*
		*	@return
		*/
		Vector2D
			rotated(
			double angle
			) const;

		/**
		*
		*	@return
		*/
		Vector2D&
			rotate(
			double angle
			);

		Vector2D
			projection(
			const Vector2D& onto
			) const;

		/**
		*
		*	@param col
		*	@return
		*/
		const Vector2D&
			draw(
			const Color& color = Palette::White
			) const;

		/**
		*	Vector2D{ 0, 0 }
		*/
		const static Vector2D Zero;

		/**
		*	Vector2D{ 1, 1 }
		*/
		const static Vector2D One;

		/**
		*	Vector2D{ 1, 0 }
		*/
		const static Vector2D UnitX;

		/**
		*	Vector2D{ 0, 1 }
		*/
		const static Vector2D UnitY;

		/**
		*	Vector2D{ -1, 0 }
		*/
		const static Vector2D Left;

		/**
		*	Vector2D{ 1, 0 }
		*/
		const static Vector2D Right;

		/**
		*	Vector2D{ 0, -1 }
		*/
		const static Vector2D Up;

		/**
		*	Vector2D{ 0, 1 }
		*/
		const static Vector2D Down;
	};

	/**
	*
	*	@param os
	*	@param v
	*	@return
	*/
	template<typename CharType, typename Type>
	inline
	std::basic_ostream<CharType>&
		operator << (
		std::basic_ostream<CharType>& os,
		const Vector2D<Type>& v
		)
	{
		return os << CharType('(') << v.x << CharType(',') << v.y << CharType(')');
	}

	/**
	*
	*	@param is
	*	@param v
	*	@return
	*/
	template<typename CharType, typename Type>
	inline
	std::basic_istream<CharType>&
		operator >> (
		std::basic_istream<CharType>& is,
		Vector2D<Type>& v
		)
	{
		CharType unused;
		return is >> unused >> v.x >> unused >> v.y >> unused;
	}

	using Float2	= Vector2D<float>;
	using Vec2		= Vector2D<double>;

	/**
	*
	*	@param s
	*	@param v
	*	@return
	*/
	template<typename Type, typename U>
	inline Vector2D<Type> operator * (U s, const Vector2D<Type>& v)
	{
		return v*s;
	}

	/**
	*
	*	@param p
	*	@param v
	*	@return
	*/
	template<typename Type>
	inline Vector2D<Type> operator + (const Point& p, const Vector2D<Type>& v)
	{
		return v + p;
	}

	/**
	*
	*	@param p
	*	@param v
	*	@return
	*/
	template<typename Type>
	inline Vector2D<Type> operator - (const Point& p, const Vector2D<Type>& v)
	{
		return Vector2D<Type>(p)-v;
	}
}
