﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <iostream>
# include "SivPropertyMacro.hpp"
# include "SivPointVector.hpp"

namespace s3d
{
	/**
	*	@brief 3 次元ベクトル
	*/
	template<typename Type>
	struct Vector3D
	{
		using value_type = Type;

		Type x, y, z;

		/**
		*
		*/
		Vector3D() {}

		/**
		*
		*	@param _x
		*	@param _y
		*	@param _z
		*/
		template<typename U, typename V, typename W>
		Vector3D(U _x, V _y, W _z) : x(static_cast<Type>(_x)), y(static_cast<Type>(_y)), z(static_cast<Type>(_z)) {}

		/**
		*
		*	@param _x
		*	@param _y
		*	@param _z
		*/
		Vector3D(Type _x, Type _y, Type _z) : x(_x), y(_y), z(_z) {}

		/**
		*
		*	@param xy
		*	@param _z
		*/
		template<typename U>
		Vector3D(const Vector2D<U>& xy, Type _z) : x(static_cast<Type>(xy.x)), y(static_cast<Type>(xy.y)), z(_z) {}

		/**
		*
		*	@param _x
		*	@param yz
		*/
		template<typename U>
		Vector3D(Type _x, const Vector2D<U>& yz) : x(_x), y(static_cast<Type>(yz.x)), z(static_cast<Type>(yz.y)) {}

		/**
		*
		*	@param v
		*/
		template<typename U>
		Vector3D(const Vector3D<U>& v) : x(static_cast<Type>(v.x)), y(static_cast<Type>(v.y)), z(static_cast<Type>(v.z)) {}

		/**
		*
		*	@return
		*/
		Vector3D operator + () const;

		/**
		*
		*	@return
		*/
		Vector3D operator - () const;

		/**
		*
		*	@param v
		*	@return
		*/
		Vector3D operator + (const Vector3D& v) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Vector3D operator - (const Vector3D& v) const;

		/**
		*
		*	@param s
		*	@return
		*/
		Vector3D operator * (Type s) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Vector3D operator * (const Vector3D& v) const;

		/**
		*
		*	@param s
		*	@return
		*/
		Vector3D operator / (Type s) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Vector3D& operator += (const Vector3D& v);

		/**
		*
		*	@param v
		*	@return
		*/
		Vector3D& operator -= (const Vector3D& v);

		/**
		*
		*	@param s
		*	@return
		*/
		Vector3D& operator *= (Type s);

		/**
		*
		*	@param s
		*	@return
		*/
		Vector3D& operator /= (Type s);

		/**
		*
		*/
		Property_Get(Vector2D<Type>, xy) const;

		/**
		*
		*/
		Property_Get(Vector2D<Type>, xz) const;

		/**
		*
		*/
		Property_Get(Vector2D<Type>, yz) const;

		/**
		*
		*	@param _x
		*	@param _y
		*	@param _z
		*	@return
		*/
		Vector3D& set(Type _x, Type _y, Type _z) { x = _x; y = _y; z = _z; return *this; }

		/**
		*
		*	@param v
		*	@return
		*/
		Vector3D& set(const Vector3D& v) { return *this = v; }

		/**
		*
		*	@param _x
		*	@param _y
		*	@return
		*/
		Vector3D movedBy(Type _x, Type _y, Type _z) const { return{ x + _x, y + _y, z + _z }; }

		/**
		*
		*	@param v
		*	@return
		*/
		Vector3D movedBy(const Vector3D& v) const { return{ x + v.x, y + v.y, z + v.z }; }

		/**
		*
		*	@param _x
		*	@param _y
		*	@param _z
		*	@return
		*/
		Vector3D& moveBy(Type _x, Type _y, Type _z) { x += _x; y += _y; z += _z; return *this; }

		/**
		*
		*	@param v
		*	@return
		*/
		Vector3D& moveBy(const Vector3D& v) { return *this += v; }

		/**
		*
		*/
		Property_Get(bool, isZero) const;

		/**
		*
		*	@return
		*/
		Type
			length(
			) const;

		/**
		*
		*	@return
		*/
		Type
			lengthSq(
			) const;

		/**
		*
		*	@return
		*/
		Type
			lengthInv(
			) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Type
			distanceFrom(
			const Vector3D& v
			) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Type
			distanceFromSq(
			const Vector3D& v
			) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Type
			dot(
			const Vector3D& v
			) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Vector3D
			cross(
			const Vector3D& v
			) const;

		/**
		*
		*	@return
		*/
		Vector3D
			normalized(
			) const;

		/**
		*
		*	@return
		*/
		Vector3D&
			normalize(
			);

		/**
		*
		*	@param primitive
		*	@return
		*/
		template<typename Primitive>
		bool intersects(const Primitive& primitive) const
		{
			return Geometry3D::Intersect(*this, primitive);
		}

		/**
		*	Vector3D{ 0, 0, 0 }
		*/
		const static Vector3D Zero;

		/**
		*	Vector3D{ 1, 1, 1 }
		*/
		const static Vector3D One;

		/**
		*	Vector3D{ 1, 0, 0 }
		*/
		const static Vector3D UnitX;

		/**
		*	Vector3D{ 0, 1, 0 }
		*/
		const static Vector3D UnitY;

		/**
		*	Vector3D{ 0, 0, 1 }
		*/
		const static Vector3D UnitZ;

		/**
		*	Vector3D{ -1, 0, 0 }
		*/
		const static Vector3D Left;

		/**
		*	Vector3D{ 1, 0, 0 }
		*/
		const static Vector3D Right;

		/**
		*	Vector3D{ 0, 1, 0 }
		*/
		const static Vector3D Up;

		/**
		*	Vector3D{ 0, -1, 0 }
		*/
		const static Vector3D Down;

		/**
		*	Vector3D{ 0, 0, 1 }
		*/
		const static Vector3D Forward;

		/**
		*	Vector3D{ 0, 0, -1 }
		*/
		const static Vector3D Backward;
	};

	/**
	*
	*	@param os
	*	@param v
	*	@return
	*/
	template<typename CharType, typename Type>
	inline
		std::basic_ostream<CharType>&
		operator << (
		std::basic_ostream<CharType>& os,
		const Vector3D<Type>& v
		)
	{
		return os << CharType('(') << v.x << CharType(',') << v.y << CharType(',') << v.z << CharType(')');
	}

	/**
	*
	*	@param is
	*	@param v
	*	@return
	*/
	template<typename CharType, typename Type>
	inline
		std::basic_istream<CharType>&
		operator >> (
		std::basic_istream<CharType>& is,
		Vector3D<Type>& v
		)
	{
		CharType unused;
		return is >> unused >> v.x >> unused >> v.y >> unused >> v.z >> unused;
	}

	using Float3	= Vector3D<float>;
	using Vec3		= Vector3D<double>;

	/**
	*
	*	@param s
	*	@param v
	*	@return
	*/
	template<typename Type, typename U>
	inline Vector3D<Type> operator * (U s, const Vector3D<Type>& v)
	{
		return v*s;
	}
}
