﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <iostream>
# include "SivPropertyMacro.hpp"

namespace s3d
{
	/**
	*	@brief 4 次元ベクトル
	*/
	template<typename Type>
	struct Vector4D
	{
		using value_type = Type;

		Type x, y, z, w;

		/**
		*
		*/
		Vector4D(){}

		/**
		*
		*	@param _x
		*	@param _y
		*	@param _z
		*	@param _w
		*/
		template<typename U, typename V, typename W, typename X>
		Vector4D(U _x, V _y, W _z, X _w) : x(static_cast<Type>(_x)), y(static_cast<Type>(_y)), z(static_cast<Type>(_z)), w(static_cast<Type>(_w)){}

		/**
		*
		*	@param _x
		*	@param _y
		*	@param _z
		*	@param _w
		*/
		Vector4D(Type _x, Type _y, Type _z, Type _w) : x(_x), y(_y), z(_z), w(_w){}

		/**
		*
		*	@param xy
		*	@param _z
		*	@param _w
		*/
		template<typename U>
		Vector4D(const Vector2D<U>& xy, Type _z, Type _w) : x(static_cast<Type>(xy.x)), y(static_cast<Type>(xy.y)), z(_z), w(_w){}

		/**
		*
		*	@param _x
		*	@param _y
		*	@param zw
		*/
		template<typename U>
		Vector4D(Type _x, Type _y, const Vector2D<U>& zw) : x(_x), y(_y), z(static_cast<Type>(zw.x)), w(static_cast<Type>(zw.y)){}

		/**
		*
		*	@param xy
		*	@param zw
		*/
		template<typename U, typename V>
		Vector4D(const Vector2D<U>& xy, const Vector2D<V>& zw) : x(static_cast<Type>(xy.x)), y(static_cast<Type>(xy.y)), z(static_cast<Type>(zw.x)), w(static_cast<Type>(zw.y)){}

		/**
		*
		*	@param xyz
		*	@param _w
		*/
		template<typename U>
		Vector4D(const Vector3D<U>& xyz, Type _w) : x(static_cast<Type>(xyz.x)), y(static_cast<Type>(xyz.y)), z(static_cast<Type>(xyz.z)), w(_w){}

		/**
		*
		*	@param _x
		*	@param yzw
		*/
		template<typename U>
		Vector4D(Type _x, const Vector3D<U>& yzw) : x(_x), y(static_cast<Type>(yzw.y)), z(static_cast<Type>(yzw.y)), w(static_cast<Type>(yzw.z)){}

		/**
		*
		*	@param v
		*/
		template<typename U>
		Vector4D(const Vector4D<U>& v) : x(static_cast<Type>(v.x)), y(static_cast<Type>(v.y)), z(static_cast<Type>(v.z)), w(static_cast<Type>(v.w)){}

		/**
		*
		*	@return
		*/
		Vector4D operator + () const;

		/**
		*
		*	@return
		*/
		Vector4D operator - () const;

		/**
		*
		*	@param v
		*	@return
		*/
		Vector4D operator + (const Vector4D& v) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Vector4D operator - (const Vector4D& v) const;

		/**
		*
		*	@param s
		*	@return
		*/
		Vector4D operator * (Type s) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Vector4D operator * (const Vector4D& v) const;

		/**
		*
		*	@param s
		*	@return
		*/
		Vector4D operator / (Type s) const;

		/**
		*
		*	@param v
		*	@return
		*/
		Vector4D& operator += (const Vector4D& v);

		/**
		*
		*	@param v
		*	@return
		*/
		Vector4D& operator -= (const Vector4D& v);

		/**
		*
		*	@param s
		*	@return
		*/
		Vector4D& operator *= (Type s);

		/**
		*
		*	@param s
		*	@return
		*/
		Vector4D& operator /= (Type s);

		/**
		*
		*/
		Property_Get(Vector2D<Type>, xy) const;

		/**
		*
		*/
		Property_Get(Vector2D<Type>, yz) const;

		/**
		*
		*/
		Property_Get(Vector2D<Type>, zw) const;

		/**
		*
		*/
		Property_Get(Vector3D<Type>, xyz) const;

		/**
		*
		*/
		Property_Get(Vector3D<Type>, yzw) const;

		/**
		*
		*	@param _x
		*	@param _y
		*	@param _z
		*	@param _w
		*	@return
		*/
		Vector4D& set(Type _x, Type _y, Type _z, Type _w) { x = _x; y = _y; z = _z; w = _w; return *this; }

		/*
		*
		*	@param v
		*	@return
		*/
		Vector4D& set(const Vector4D& v) { return *this = v; }

		/**
		*
		*	@param _x
		*	@param _y
		*	@return
		*/
		Vector4D movedBy(Type _x, Type _y, Type _z, Type _w) const { return{ x + _x, y + _y, z + _z, w + _w }; }

		/**
		*
		*	@param v
		*	@return
		*/
		Vector4D movedBy(const Vector4D& v) const { return{ x + v.x, y + v.y, z + v.z, w + v.w }; }

		/**
		*
		*	@param _x
		*	@param _y
		*	@param _z
		*	@param _w
		*	@return
		*/
		Vector4D& moveBy(Type _x, Type _y, Type _z, Type _w) { x += _x; y += _y; z += _z; w += _w; return *this; }

		/*
		*
		*	@param v
		*	@return
		*/
		Vector4D& moveBy(const Vector4D& v) { return *this += v; }

		/**
		*
		*/
		Property_Get(bool, isZero) const;

		/**
		*
		*	@return
		*/
		Type
			length(
			) const;

		/**
		*
		*	@return
		*/
		Type
			lengthSq(
			) const;

		/**
		*
		*	@return
		*/
		Type
			lengthInv(
			) const;

		/*
		*
		*	@param v
		*	@return
		*/
		Type
			distanceFrom(
			const Vector4D& v
			) const;

		/*
		*
		*	@param v
		*	@return
		*/
		Type
			distanceFromSq(
			const Vector4D& v
			) const;

		/*
		*
		*	@param v
		*	@return
		*/
		Type
			dot(
			const Vector4D& v
			) const;

		/*
		*
		*	@return
		*/
		Vector4D
			normalized(
			) const;

		/*
		*
		*	@return
		*/
		Vector4D&
			normalize(
			);

		/**
		*	Vector4D{ 0, 0, 0, 0 }
		*/
		const static Vector4D Zero;

		/**
		*	Vector4D{ 1, 1, 1, 1 }
		*/
		const static Vector4D One;

		/**
		*	Vector4D{ 1, 0, 0, 0 }
		*/
		const static Vector4D UnitX;

		/**
		*	Vector4D{ 0, 1, 0, 0 }
		*/
		const static Vector4D UnitY;

		/**
		*	Vector4D{ 0, 0, 1, 0 }
		*/
		const static Vector4D UnitZ;

		/**
		*	Vector4D{ 0, 0, 0, 1 }
		*/
		const static Vector4D UnitW;
	};

	/**
	*
	*	@param os
	*	@param v
	*	@return
	*/
	template<typename CharType, typename Type>
	inline
		std::basic_ostream<CharType>&
		operator << (
		std::basic_ostream<CharType>& os,
		const Vector4D<Type>& v
		)
	{
		return os << CharType('(') << v.x << CharType(',') << v.y << CharType(',') << v.z << CharType(',') << v.w << CharType(')');
	}

	/**
	*
	*	@param is
	*	@param v
	*	@return
	*/
	template<typename CharType, typename Type>
	inline
		std::basic_istream<CharType>&
		operator >> (
		std::basic_istream<CharType>& is,
		Vector4D<Type>& v
		)
	{
		CharType unused;
		return is >> unused >> v.x >> unused >> v.y >> unused >> v.z >> unused >> v.w >> unused;
	}

	using Float4	= Vector4D<float>;
	using Vec4		= Vector4D<double>;

	/**
	*
	*	@param s
	*	@param v
	*	@return
	*/
	template<typename Type, typename U>
	inline Vector4D<Type> operator * (U s, const Vector4D<Type>& v)
	{
		return v*s;
	}
}
