﻿//-----------------------------------------------
//
//	This file is part of the Siv3D Engine.
//
//	Copyright (C) 2008-2014 Ryo Suzuki
//
//	Licensed under the MIT License.
//
//-----------------------------------------------

# pragma once
# include <memory>
# include "Siv3DFwd.hpp"
# include "SivHandle.hpp"
# include "SivPropertyMacro.hpp"

namespace s3d
{
	/// <summary>
	/// 頂点シェーダ
	/// </summary>
	/// <remarks>
	/// 将来のバージョンの Siv3D で使用されます。
	/// </remarks>
	class VertexShader
	{
	private:

		class Handle{};

		using VertexShaderHandle = Siv3DHandle<Handle>;

		std::shared_ptr<VertexShaderHandle> m_handle;

	public:

		VertexShader();

		explicit VertexShader(const FilePath& path, bool preloadShader = false);

		~VertexShader();

		void release();

		explicit operator bool() const { return !isEmpty; }

		Property_Get(HandleIDType, id) const;

		Property_Get(bool, isEmpty) const;

		bool operator == (const VertexShader& vertexShader) const;

		bool operator != (const VertexShader& vertexShader) const;

		const MemoryReader& getBinary() const;
	};
}
